Package: CarletonStats
Title: Functions for Statistics Classes at Carleton College
Version: 2.3
Authors@R: c(
  person("Laura", "Chihara" , email = "flchihara@carleton.edu", role = c("aut")),
  person("Adam", "Loy", email = "aloy@carleton.edu", role = c("aut", "cre"), 
    comment = c(ORCID = "0000-0002-5780-4611"))
   )
Description: Includes commands for bootstrapping and permutation tests, a
    command for created grouped bar plots, and a demo of the
    quantile-normal plot for data drawn from different distributions.
Depends: R (>= 3.5.0)
License: GPL-2
URL: https://github.com/aloy/CarletonStats
BugReports: https://github.com/aloy/CarletonStats/issues
Suggests: grDevices, MASS, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: TRUE
NeedsCompilation: no
RoxygenNote: 8.0.0
Imports: ggplot2 (>= 3.3.0), scales, patchwork
Packaged: 2026-08-28 14:25:31 UTC; aloy
Author: Laura Chihara [aut],
  Adam Loy [aut, cre] (ORCID: <https://orcid.org/0000-0002-5780-4611>)
Maintainer: Adam Loy <aloy@carleton.edu>
Repository: CRAN
Date/Publication: 2026-08-28 17:00:16 UTC

# Generated by roxygen2: do not edit by hand

S3method(boot,default)
S3method(boot,formula)
S3method(bootCor,default)
S3method(bootCor,formula)
S3method(bootPaired,default)
S3method(bootPaired,formula)
S3method(bootSlope,default)
S3method(bootSlope,formula)
S3method(confint,carlboot)
S3method(groupedBar,default)
S3method(groupedBar,formula)
S3method(permTest,default)
S3method(permTest,formula)
S3method(permTestAnova,default)
S3method(permTestAnova,formula)
S3method(permTestCor,default)
S3method(permTestCor,formula)
S3method(permTestPaired,default)
S3method(permTestPaired,formula)
S3method(permTestSlope,default)
S3method(permTestSlope,formula)
S3method(plot,carlboot)
S3method(plot,carlperm)
S3method(print,carlboot)
S3method(print,carlperm)
S3method(stemPlot,default)
S3method(stemPlot,formula)
S3method(summary,carlboot)
S3method(summary,carlperm)
export(anovaSummarized)
export(boot)
export(bootCor)
export(bootPaired)
export(bootSlope)
export(confIntDemo)
export(corDemo)
export(groupedBar)
export(missingLevel)
export(permTest)
export(permTestAnova)
export(permTestCor)
export(permTestPaired)
export(permTestSlope)
export(qqPlotDemo)
export(stemPlot)
import(ggplot2)
importFrom(grDevices,dev.new)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,stem)
importFrom(graphics,title)
importFrom(patchwork,wrap_plots)
importFrom(scales,label_percent)
importFrom(stats,aov)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,pf)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,rchisq)
importFrom(stats,rexp)
importFrom(stats,rf)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)

<!-- badges: start -->

[![R-CMD-check](https://github.com/aloy/CarletonStats/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/aloy/CarletonStats/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

# CarletonStats

<!-- badges: start -->

<!-- badges: end -->

The **CarletonStats** package includes commands for bootstrapping and
permutation tests used in Stat 120 at Carleton College. Additional
commands of interest include a command for creating grouped bar plots,
and a demo of the quantile-normal plot for data drawn from different
distributions.

## Installation

You can install the development version of CarletonStats from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("aloy/CarletonStats")
```

#' @param call the result of `match.call(expand.dots = FALSE)` from the caller
#' @param parent_env the caller's `parent.frame()`
#' @return A list with elements `x`, `y`, `x.name`, `y.name`
#' @noRd
.parse_formula_two_var <- function(call, parent_env) {
  m <- call
  if (is.matrix(eval(m$data, parent_env))) {
    m$data <- as.data.frame(eval(m$data, parent_env))
  }
  m[[1L]] <- as.name("model.frame")
  m$... <- NULL
  mf <- eval(m, parent_env)

  if (length(mf) != 2L) {
    stop("Invalid formula")
  }

  nmiss <- length(attr(mf, "na.action"))
  if (nmiss > 0) {
    cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
  }

  varnames <- names(mf)
  response <- attr(attr(mf, "terms"), "response")
  y <- mf[[response]]
  x <- mf[[-response]]

  list(x = x, y = y, x.name = varnames[2], y.name = varnames[1])
}

.encode_binary <- function(x, success = NULL) {
  if (is.character(x)) {
    x <- factor(x)
  }

  if (is.logical(x)) {
    success_label <- if (is.null(success)) "TRUE" else as.character(success)
    if (!success_label %in% c("FALSE", "TRUE")) {
      stop("'success' must be \"TRUE\" or \"FALSE\" for logical variables")
    }
    x <- as.integer(x == (success_label == "TRUE"))
  } else if (is.factor(x)) {
    x <- droplevels(x)
    lvls <- levels(x)
    if (length(lvls) != 2L) {
      stop("Categorical variable must have exactly 2 levels")
    }
    success_label <- if (is.null(success)) lvls[2] else as.character(success)
    if (!success_label %in% lvls) {
      stop(paste0(
        "'success' (\"",
        success_label,
        "\") is not a level of x. ",
        "Valid levels: ",
        paste(lvls, collapse = ", ")
      ))
    }
    x <- as.integer(x == success_label)
  } else {
    stop("Variable must be numeric, logical, factor, or character.")
  }

  message("Note: \"", success_label, "\" coded as the success (1).")
  list(x = x, success = success_label)
}

#' @describeIn permTest Permutation test
#' @export

permTest.default <- function(
  x,
  group,
  statistic = mean,
  success = NULL,
  B = 9999,
  alternative = "two.sided",
  plot.hist = TRUE,
  plot.qq = FALSE,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
) {
  if (!is.null(seed)) {
    set.seed(seed)
  }

  stat <- match.fun(statistic)

  is_proportion <- FALSE
  if (!is.numeric(x)) {
    enc <- .encode_binary(x, success)
    x <- enc$x
    is_proportion <- TRUE
  } else if (!is.null(success)) {
    warning("'success' is ignored when x is numeric.")
  }
  if (is.factor(group)) {
    group <- droplevels(group)
  } else {
    group <- as.factor(group)
  }
  if (length(levels(group)) != 2) {
    stop("Grouping variable must have only two levels")
  }

  comCases <- complete.cases(x, group)
  nmiss <- length(x) - sum(comCases)
  if (nmiss > 0) {
    cat("\n ", nmiss, "observations removed due to missing values.\n")
  }

  x <- x[comCases]
  group <- group[comCases]

  group1 <- x[group == levels(group)[1]]
  group2 <- x[group == levels(group)[2]]
  group1.name <- levels(group)[1]
  group2.name <- levels(group)[2]

  stat1 <- round(stat(group1), 5)
  stat2 <- round(stat(group2), 5)
  n <- length(x)
  m <- length(group1)

  observed <- stat(group1) - stat(group2)

  result <- numeric(B)
  for (i in 1:B) {
    index <- sample(n, m, replace = FALSE)
    result[i] <- stat(x[index]) - stat(x[-index])
  } #end for

  alt <- pmatch(alternative, c("less", "greater", "two.sided"), nomatch = 4)

  P.value <- .calc_pvalue(result, observed, alt)

  if (P.value > 1) {
    P.value <- 1
  }

  class(result) <- "carlperm"
  attr(result, "observed") <- observed
  attr(result, "statistic") <- if (is_proportion) "proportion" else as.character(substitute(statistic))
  attr(result, "alternative") <- alternative
  attr(result, "groups") <- levels(group)
  attr(result, "group.stats") <- c(stat(group1), stat(group2))
  attr(result, "pval") <- P.value
  attr(result, "xlab") <- xlab
  attr(result, "ylab") <- ylab
  attr(result, "title") <- title
  attr(result, "plot.hist") <- plot.hist
  attr(result, "plot.qq") <- plot.qq

  result
}

#' Permutation test
#'
#' Permutation test to test a hypothesis involving two samples.
#'
#' Permutation test to see if a population parameter is the same for two
#' populations. For instance, test \eqn{latex}{H_0: \mu_M = \mu_F} where
#' \eqn{latex}{\mu} denotes the population mean. The values of the numeric
#' variable are randomly assigned to the two groups and the difference of the
#' statistic for each group is calculated. The command will print the mean and
#' standard error of the distribution of the test statistic as well as a
#' P-value.
#'
#' Observations with missing values are removed.
#'
#' @aliases permTest permTest.default permTest.formula
#' @param x a numeric, logical, factor, or character vector. Logical, factor,
#' and character vectors with exactly two unique values are converted to 0/1,
#' and \code{mean} is used to compute the proportion.
#' @param group a factor variable with two levels. If \code{group} is a binary
#' numeric vector, it will be coerced into a factor variable.
#' @param statistic the statistic of interest.
#' @param success a character string naming the level of \code{x} to code as
#' 1 when \code{x} is a logical, factor, or character variable. Defaults to
#' \code{NULL}, which uses the second factor level (alphabetically) or
#' \code{TRUE} for logical vectors.
#' @param B the number of resamples (positive integer greater than 2).
#' @param seed optional argument to \code{\link{set.seed}}
#' @param alternative the alternative hypothesis. Options are
#' \code{"two.sided"}, \code{"less"} or \code{"greater"}.
#' @param plot.hist a logical value. If \code{TRUE}, the permutation
#' distribution of the statistic is plotted.
#' @param plot.qq a logical value. If \code{TRUE}, then a normal
#' quantile-quantile plot of the resampled test statistic is created.
#' @param formula a formula of the form \code{y ~ group} where \code{y} is
#' numeric and \code{group} is a factor variable.
#' @param data a data frame with the variables in the formula.
#' @param subset an optional expression specifying which observations to keep.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param \dots further arguments to be passed to or from methods.
#' @return Returns invisibly a vector of the replicates of the test statistic.
#' @author Laura Chihara
#' @references Tim Hesteberg's website:
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords permutation test resampling randomization
#' @examples
#'
#' # Testing the difference in means
#' permTest(states03$ViolentCrime, states03$DeathPenalty)
#'
#' #using formula syntax
#' permTest(ViolentCrime ~ DeathPenalty, data = states03, alt = "less")
#'
#' # Testing the difference in proportions
#' permTest(penguin_survival$Status, penguin_survival$TagType, B = 999)
#'
#' #using formula syntax
#' permTest(Status ~ TagType, data = penguin_survival, B = 999)
#'
#' @export

permTest <-
  function(x, ...) {
    UseMethod("permTest")
  }

#' @title Print a summary of an \code{carlboot} object
#'
#' @description
#' Print summary statistics and confidence intervals, if desired, for an \code{lmeresamp} object.
#'
#' @param object The carlboot object to print.
#' @param ... not used
#'
#' @rdname summary
#' @export
#' @method summary carlboot
#' @examples
#' boot_dist <- boot(ToothGrowth$len, ToothGrowth$supp, B = 1000)
#' summary(boot_dist)
summary.carlboot <- function(object, ...) {
  cat("Replications:", length(object), "\n\n")

  cat("Summary Statistics of Bootstrap Distribution:", "\n")

  print(
    data.frame(
      Observed = attr(object, "observed"),
      Mean = mean(object),
      SE = sd(object)
    )
  )
}


#' @rdname summary
#' @export
#' @method summary carlperm
#' @examples
#' perm_dist <- permTest(states03$ViolentCrime, states03$DeathPenalty, B = 999)
#' summary(perm_dist)
summary.carlperm <- function(object, ...) {
  alt <- pmatch(
    attr(object, "alternative"),
    c("less", "greater", "two.sided"),
    nomatch = 4
  )

  cat("Replications:", length(object), "\n\n")

  cat("Summary Statistics of Permutation Distribution:", "\n\n")

  res <- data.frame(
    Observed = attr(object, "observed"),
    Mean = mean(object),
    Alternative = attr(object, "alternative"),
    "P.value" = .calc_pvalue(object, attr(object, "observed"), alt)
  )

  if (attr(object, "statistic") == "F") {
    res$df1 <- attr(object, "df")[1]
    res$df2 <- attr(object, "df")[2]
    res <- res[, c("Observed", "Mean", "df1", "df2", "P.value")]
  }

  print(res)
}

## boot() ---------------------------------------------------------------

test_that("boot() single variable returns carlboot with correct attributes", {
  result <- boot(ToothGrowth$len, B = 500, seed = 1, plot.hist = FALSE)
  expect_s3_class(result, "carlboot")
  expect_length(result, 500)
  expect_equal(attr(result, "statistic"), "mean")
  expect_null(attr(result, "groups"))
})

test_that("boot() two-group returns correct observed difference", {
  result <- boot(
    ToothGrowth$len,
    ToothGrowth$supp,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  obs <- mean(ToothGrowth$len[ToothGrowth$supp == "OJ"]) -
    mean(ToothGrowth$len[ToothGrowth$supp == "VC"])
  expect_equal(attr(result, "observed"), obs)
})

test_that("boot() formula interface matches default interface", {
  r1 <- boot(
    len ~ supp,
    data = ToothGrowth,
    B = 200,
    seed = 42,
    plot.hist = FALSE
  )
  r2 <- boot(
    ToothGrowth$len,
    ToothGrowth$supp,
    B = 200,
    seed = 42,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

test_that("boot() print output is stable", {
  result <- boot(ToothGrowth$len, B = 500, seed = 1, plot.hist = FALSE)
  expect_snapshot(print(result))
})

test_that("boot() two-group print output is stable", {
  result <- boot(
    ToothGrowth$len,
    ToothGrowth$supp,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

## bootPaired() ----------------------------------------------------------

test_that("bootPaired() statistic attribute is 'mean'", {
  result <- bootPaired(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "mean")
})

test_that("bootPaired() formula and default interfaces match", {
  r1 <- bootPaired(
    VanillaCalories ~ ChocCalories,
    data = Icecream,
    B = 200,
    seed = 7,
    plot.hist = FALSE
  )
  r2 <- bootPaired(
    Icecream$ChocCalories,
    Icecream$VanillaCalories,
    B = 200,
    seed = 7,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

test_that("bootPaired() print output is stable", {
  result <- bootPaired(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

## bootCor() -------------------------------------------------------------

test_that("bootCor() statistic attribute is 'correlation'", {
  result <- bootCor(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "correlation")
})

test_that("bootCor() observed matches cor()", {
  result <- bootCor(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(
    attr(result, "observed"),
    cor(Icecream$VanillaCalories, Icecream$ChocCalories)
  )
})

test_that("bootCor() formula and default interfaces match", {
  r1 <- bootCor(
    VanillaCalories ~ ChocCalories,
    data = Icecream,
    B = 200,
    seed = 3,
    plot.hist = FALSE
  )
  r2 <- bootCor(
    Icecream$ChocCalories,
    Icecream$VanillaCalories,
    B = 200,
    seed = 3,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

test_that("bootCor() print output is stable", {
  result <- bootCor(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

## bootSlope() -----------------------------------------------------------

test_that("bootSlope() statistic attribute is 'slope'", {
  result <- bootSlope(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "slope")
})

test_that("bootSlope() observed matches manual calculation", {
  x <- Icecream$VanillaCalories
  y <- Icecream$ChocCalories
  expected <- round(cor(x, y) * sd(y) / sd(x), 4)
  result <- bootSlope(x, y, B = 500, seed = 1, plot.hist = FALSE)
  expect_equal(attr(result, "observed"), expected)
})

test_that("bootSlope() print output is stable", {
  result <- bootSlope(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 500,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("bootSlope() formula and default interfaces match", {
  r1 <- bootSlope(
    VanillaCalories ~ ChocCalories,
    data = Icecream,
    B = 200,
    seed = 3,
    plot.hist = FALSE
  )
  r2 <- bootSlope(
    Icecream$ChocCalories,
    Icecream$VanillaCalories,
    B = 200,
    seed = 3,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

## permTest() ------------------------------------------------------------

test_that("permTest() returns carlperm with p-value in [0, 1]", {
  result <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_s3_class(result, "carlperm")
  expect_true(attr(result, "pval") >= 0 && attr(result, "pval") <= 1)
})

test_that("permTest() formula and default interfaces match", {
  r1 <- permTest(
    ViolentCrime ~ DeathPenalty,
    data = states03,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  r2 <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

test_that("permTest() print output is stable", {
  result <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("boot() categorical single variable returns proportion statistic", {
  x <- c("yes", "no", "yes", "yes", "no", "yes")
  result <- boot(x, success = "yes", B = 500, seed = 1, plot.hist = FALSE)
  expect_s3_class(result, "carlboot")
  expect_equal(attr(result, "statistic"), "proportion")
  expect_equal(attr(result, "observed"), mean(x == "yes"))
})

test_that("boot() categorical two-group uses proportion statistic", {
  x <- c("yes", "no", "yes", "no", "yes", "no")
  g <- c("A", "A", "A", "B", "B", "B")
  result <- boot(x, g, success = "yes", B = 500, seed = 1, plot.hist = FALSE)
  expect_equal(attr(result, "statistic"), "proportion")
  expected_obs <- mean(x[g == "A"] == "yes") - mean(x[g == "B"] == "yes")
  expect_equal(attr(result, "observed"), expected_obs)
})

test_that("boot() logical vector uses TRUE as default success", {
  x <- c(TRUE, FALSE, TRUE, TRUE, FALSE)
  result <- boot(x, B = 500, seed = 1, plot.hist = FALSE)
  expect_equal(attr(result, "statistic"), "proportion")
  expect_equal(attr(result, "observed"), mean(x))
})

test_that("boot() factor variable uses second level as default success", {
  x <- factor(c("no", "yes", "yes", "no", "yes"))
  result <- boot(x, B = 500, seed = 1, plot.hist = FALSE)
  expect_equal(attr(result, "statistic"), "proportion")
  expect_equal(attr(result, "observed"), mean(x == "yes"))
})

test_that("boot() errors when success is not a valid level", {
  x <- c("yes", "no", "yes")
  expect_error(
    boot(x, success = "maybe", B = 100, seed = 1, plot.hist = FALSE),
    "not a level"
  )
})

test_that("boot() warns when success is passed with numeric x", {
  expect_warning(
    boot(ToothGrowth$len, success = "A", B = 100, seed = 1, plot.hist = FALSE),
    "ignored"
  )
})

test_that("boot() categorical print output is stable", {
  x <- c("yes", "no", "yes", "yes", "no", "yes", "no", "yes")
  g <- c("A", "A", "A", "A", "B", "B", "B", "B")
  result <- boot(x, g, success = "yes", B = 500, seed = 1, plot.hist = FALSE)
  expect_snapshot(print(result))
})

test_that("permTest() categorical variable returns proportion statistic", {
  x <- c("yes", "no", "yes", "no", "yes", "no", "yes", "no")
  g <- c("A", "A", "A", "A", "B", "B", "B", "B")
  result <- permTest(x, g, success = "yes", B = 499, seed = 1, plot.hist = FALSE)
  expect_s3_class(result, "carlperm")
  expect_equal(attr(result, "statistic"), "proportion")
})

test_that("permTest() categorical print output is stable", {
  x <- c("yes", "no", "yes", "no", "yes", "no", "yes", "no")
  g <- c("A", "A", "A", "A", "B", "B", "B", "B")
  result <- permTest(x, g, success = "yes", B = 499, seed = 1, plot.hist = FALSE)
  expect_snapshot(print(result))
})

test_that("permTest() warns when success is passed with numeric x", {
  expect_warning(
    permTest(
      states03$ViolentCrime, states03$DeathPenalty,
      success = "A", B = 99, seed = 1, plot.hist = FALSE
    ),
    "ignored"
  )
})

test_that("boot() single proportion observed matches manual calculation", {
  result <- boot(
    penguin_survival$Status,
    success = "Survived", B = 500, seed = 1, plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "proportion")
  expect_equal(attr(result, "observed"), mean(penguin_survival$Status == "Survived"))
})

test_that("boot() single proportion default success uses second factor level", {
  result <- boot(
    penguin_survival$Status,
    B = 500, seed = 1, plot.hist = FALSE
  )
  expected_success <- levels(factor(penguin_survival$Status))[2]
  expect_equal(attr(result, "observed"), mean(penguin_survival$Status == expected_success))
})

test_that("boot() two-group proportion observed matches manual calculation", {
  result <- boot(
    penguin_survival$Status, penguin_survival$TagType,
    success = "Survived", B = 500, seed = 1, plot.hist = FALSE
  )
  electronic <- penguin_survival$Status[penguin_survival$TagType == "Electronic"]
  metal <- penguin_survival$Status[penguin_survival$TagType == "Metal"]
  expected_obs <- mean(electronic == "Survived") - mean(metal == "Survived")
  expect_equal(attr(result, "observed"), expected_obs)
})

test_that("boot() proportion formula and default interfaces match", {
  r1 <- boot(
    Status ~ TagType, data = penguin_survival,
    success = "Survived", B = 200, seed = 7, plot.hist = FALSE
  )
  r2 <- boot(
    penguin_survival$Status, penguin_survival$TagType,
    success = "Survived", B = 200, seed = 7, plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

test_that("permTest() proportion observed matches manual calculation", {
  result <- permTest(
    penguin_survival$Status, penguin_survival$TagType,
    success = "Survived", B = 999, seed = 1, plot.hist = FALSE
  )
  electronic <- penguin_survival$Status[penguin_survival$TagType == "Electronic"]
  metal <- penguin_survival$Status[penguin_survival$TagType == "Metal"]
  expected_obs <- mean(electronic == "Survived") - mean(metal == "Survived")
  expect_equal(attr(result, "observed"), expected_obs)
})

test_that("permTest() proportion formula and default interfaces match", {
  r1 <- permTest(
    Status ~ TagType, data = penguin_survival,
    success = "Survived", B = 499, seed = 3, plot.hist = FALSE
  )
  r2 <- permTest(
    penguin_survival$Status, penguin_survival$TagType,
    success = "Survived", B = 499, seed = 3, plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

test_that("boot() penguin proportion print output is stable", {
  result <- boot(
    penguin_survival$Status, penguin_survival$TagType,
    success = "Survived", B = 500, seed = 1, plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("permTest() penguin proportion print output is stable", {
  result <- permTest(
    penguin_survival$Status, penguin_survival$TagType,
    success = "Survived", B = 999, seed = 1, plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("boot() difference in proportions matches for 0/1 and factor coding", {
  g <- c("A", "A", "A", "A", "B", "B", "B", "B")
  x_numeric <- c(1, 0, 1, 1, 0, 1, 0, 0)
  x_factor <- factor(ifelse(x_numeric == 1, "yes", "no"), levels = c("no", "yes"))

  r_numeric <- boot(x_numeric, g, B = 500, seed = 1, plot.hist = FALSE)
  r_factor <- boot(x_factor, g, success = "yes", B = 500, seed = 1, plot.hist = FALSE)

  expect_equal(attr(r_numeric, "observed"), attr(r_factor, "observed"))
  expect_equal(as.numeric(r_numeric), as.numeric(r_factor))
})

test_that("permTest() difference in proportions matches for 0/1 and factor coding", {
  g <- c("A", "A", "A", "A", "B", "B", "B", "B")
  x_numeric <- c(1, 0, 1, 1, 0, 1, 0, 0)
  x_factor <- factor(ifelse(x_numeric == 1, "yes", "no"), levels = c("no", "yes"))

  r_numeric <- permTest(x_numeric, g, B = 499, seed = 1, plot.hist = FALSE)
  r_factor <- permTest(x_factor, g, success = "yes", B = 499, seed = 1, plot.hist = FALSE)

  expect_equal(attr(r_numeric, "observed"), attr(r_factor, "observed"))
  expect_equal(as.numeric(r_numeric), as.numeric(r_factor))
})

## permTestPaired() ------------------------------------------------------

test_that("permTestPaired() statistic attribute is 'paired difference'", {
  result <- permTestPaired(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "paired difference")
})

test_that("permTestPaired() print output is stable", {
  result <- permTestPaired(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("permTestPaired() formula and default interfaces match", {
  r1 <- permTestPaired(
    VanillaCalories ~ ChocCalories,
    data = Icecream,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  r2 <- permTestPaired(
    Icecream$ChocCalories,
    Icecream$VanillaCalories,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

## permTestAnova() -------------------------------------------------------

test_that("permTestAnova() statistic attribute is 'F'", {
  result <- permTestAnova(
    chickwts$weight,
    chickwts$feed,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "F")
})

test_that("permTestAnova() observed matches aov()", {
  result <- permTestAnova(
    chickwts$weight,
    chickwts$feed,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expected_f <- summary(aov(weight ~ feed, data = chickwts))[[1]][["F value"]][
    1
  ]
  expect_equal(attr(result, "observed"), expected_f)
})

test_that("permTestAnova() print output is stable", {
  result <- permTestAnova(
    chickwts$weight,
    chickwts$feed,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("permTestAnova() formula and default interfaces match", {
  r1 <- permTestAnova(
    weight ~ feed,
    data = chickwts,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  r2 <- permTestAnova(
    chickwts$weight,
    chickwts$feed,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

## permTestCor() ---------------------------------------------------------

test_that("permTestCor() statistic attribute is 'correlation'", {
  result <- permTestCor(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "correlation")
})

test_that("permTestCor() observed matches cor()", {
  result <- permTestCor(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(
    attr(result, "observed"),
    round(cor(Icecream$VanillaCalories, Icecream$ChocCalories), 4)
  )
})

test_that("permTestCor() print output is stable", {
  result <- permTestCor(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("permTestCor() formula and default interfaces match", {
  r1 <- permTestCor(
    VanillaCalories ~ ChocCalories,
    data = Icecream,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  r2 <- permTestCor(
    Icecream$ChocCalories,
    Icecream$VanillaCalories,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

## permTestSlope() -------------------------------------------------------

test_that("permTestSlope() statistic attribute is 'slope'", {
  result <- permTestSlope(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_equal(attr(result, "statistic"), "slope")
})

test_that("permTestSlope() print output is stable", {
  result <- permTestSlope(
    Icecream$VanillaCalories,
    Icecream$ChocCalories,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(print(result))
})

test_that("permTestSlope() formula and default interfaces match", {
  r1 <- permTestSlope(
    VanillaCalories ~ ChocCalories,
    data = Icecream,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  r2 <- permTestSlope(
    Icecream$ChocCalories,
    Icecream$VanillaCalories,
    B = 500,
    seed = 5,
    plot.hist = FALSE
  )
  expect_equal(as.numeric(r1), as.numeric(r2))
})

## anovaSummarized() -----------------------------------------------------

test_that("anovaSummarized() output is stable", {
  N <- table(chickwts$feed)
  mn <- tapply(chickwts$weight, chickwts$feed, mean)
  stdev <- tapply(chickwts$weight, chickwts$feed, sd)
  expect_snapshot(anovaSummarized(N, mn, stdev))
})

test_that("anovaSummarized() errors on mismatched vector lengths", {
  expect_error(anovaSummarized(c(10, 20), c(1, 2, 3), c(1, 2, 3)))
})

## groupedBar() ----------------------------------------------------------

test_that("groupedBar() single variable returns invisibly", {
  expect_invisible(groupedBar(factor(mtcars$cyl)))
})

test_that("groupedBar() two-variable returns invisibly", {
  expect_invisible(groupedBar(factor(mtcars$am), factor(mtcars$cyl)))
})

test_that("groupedBar() single-variable formula returns invisibly", {
  expect_invisible(groupedBar(~cyl, data = mtcars))
})

test_that("groupedBar() two-variable formula returns invisibly", {
  expect_invisible(groupedBar(am ~ cyl, data = mtcars))
})

## plot() ----------------------------------------------------------------

test_that("plot.carlboot() returns a ggplot object", {
  result <- boot(ToothGrowth$len, B = 500, seed = 1, plot.hist = FALSE)
  expect_s3_class(plot(result), "gg")
})

test_that("plot.carlperm() returns a ggplot object", {
  result <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_s3_class(plot(result), "gg")
})

test_that("print.carlboot() with plot.hist = TRUE runs without error", {
  result <- boot(ToothGrowth$len, B = 200, seed = 1, plot.hist = TRUE)
  expect_no_error(print(result))
})

test_that("print.carlboot() with plot.hist and plot.qq runs without error", {
  result <- boot(
    ToothGrowth$len,
    B = 200,
    seed = 1,
    plot.hist = TRUE,
    plot.qq = TRUE
  )
  expect_no_error(print(result))
})

test_that("print.carlperm() with plot.hist = TRUE runs without error", {
  result <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 500,
    seed = 1,
    plot.hist = TRUE
  )
  expect_no_error(print(result))
})

test_that("print.carlperm() with plot.hist and plot.qq runs without error", {
  result <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 500,
    seed = 1,
    plot.hist = TRUE,
    plot.qq = TRUE
  )
  expect_no_error(print(result))
})

## summary() -------------------------------------------------------------

test_that("summary.carlboot() output is stable", {
  result <- boot(ToothGrowth$len, B = 500, seed = 1, plot.hist = FALSE)
  expect_snapshot(summary(result))
})

test_that("summary.carlperm() output is stable", {
  result <- permTest(
    states03$ViolentCrime,
    states03$DeathPenalty,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(summary(result))
})

test_that("summary.carlperm() includes df columns for ANOVA results", {
  result <- permTestAnova(
    chickwts$weight,
    chickwts$feed,
    B = 999,
    seed = 1,
    plot.hist = FALSE
  )
  expect_snapshot(summary(result))
})

## stemPlot() ------------------------------------------------------------

test_that("stemPlot() single variable produces output", {
  expect_output(stemPlot(states03$Births), "Stem and Leaf plot")
})

test_that("stemPlot() returns invisibly", {
  expect_invisible(stemPlot(states03$Births))
})

test_that("stemPlot() errors on non-numeric input", {
  expect_error(stemPlot(states03$Region), "variable must be numeric")
})

test_that("stemPlot() grouped variable produces output for each group", {
  expect_output(stemPlot(states03$Births, states03$Region), "Region")
})

test_that("stemPlot() formula interface works", {
  expect_output(stemPlot(Births ~ Region, data = states03), "Stem and Leaf")
})

## pvalue.R invalid alternative ------------------------------------------

test_that("permTest() errors on invalid alternative argument", {
  expect_error(
    permTest(
      states03$ViolentCrime,
      states03$DeathPenalty,
      B = 99,
      seed = 1,
      alternative = "neither",
      plot.hist = FALSE
    ),
    "Alternative not matched"
  )
})

#' Permutation test for the Slope
#'
#' Hypothesis test for a slope of a simple linear regression model.
#' The null hypothesis is that the population slope is 0.
#'
#' Perform a permutation test to test \eqn{latex}{H_0: \beta = 0}, where
#' \eqn{latex}{\beta}is the population slope. The rows of the second
#' variable are permuted and the slope is re-computed.
#'
#' The mean and standard error of the permutation distribution is printed as
#' well as a P-value.
#'
#' Observations with missing values are removed.
#'
#' @aliases permTestSlope permTestSlope.default permTestSlope.formula
#' @param x a numeric vector.
#' @param y a numeric vector.
#' @param B the number of resamples to draw (positive integer greater than 2).
#' @param seed optional argument to \code{\link{set.seed}}
#' @param alternative alternative hypothesis. Options are \code{"two.sided"},
#' \code{"less"} or \code{"greater"}.
#' @param plot.hist a logical value. If \code{TRUE}, plot the distribution of
#' the slopes obtained from each resample.
#' @param plot.qq a logical value. If \code{TRUE}, plot the normal
#' quantile-quantile plot of the slopes obtained from each resample.
#' @param x.name Label for variable x
#' @param y.name Label for variable y
#' @param formula a formula \code{y ~ x} where \code{x, y} are numeric vectors.
#' @param data a data frame that contains the variables given in the formula.
#' @param subset an optional expression indicating what observations to use.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param \dots further arguments to be passed to or from methods.
#' @return Returns invisibly a vector of the slopes obtained by the
#' randomization.
#' @author Adam Loy, Laura Chihara
#' @references Tim Hesterberg's website:
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords permutation test randomization resampling correlation
#' @examples
#'
#' plot(states03$HSGrad, states03$TeenBirths)
#' lm(HSGrad ~ TeenBirths, data = states03)
#'
#' permTestSlope(states03$HSGrad, states03$TeenBirths)
#' permTestSlope(TeenBirths ~ HSGrad, data = states03)
#'
#' @export

permTestSlope <-
  function(x, ...) {
    UseMethod("permTestSlope")
  }

#' Correlation demonstration
#'
#' For a given \code{r}, create a scatterplot of two variables with that
#' correlation.
#'
#' Demonstrate the concept of correlation by inputting a number between -1 and
#' 1 and seeing a scatter plot of two variables with that correlation. Once you
#' invoke this command, you can continue to enter values for r. Type any number
#' \eqn{latex}{abs(r)> 1 }) to exit.
#'
#' @param r a number between -1 and 1. Enter any number r, \eqn{latex}{abs(r)>
#' 1 }, to exit the interactive session[
#' @author Laura Chihara
#' @keywords Correlation
#' @examples
#'
#' \dontrun{
#' corDemo()
#' }
#'
#' @export
#'

corDemo <-
  function(r = 0) {
    if (requireNamespace("MASS", quietly = TRUE)) {
      cat("Input r. To quit, type r = 9\n")
      while (abs(r) <= 1) {
        sigma <- cbind(c(1, r), c(r, 1))

        temp <- MASS::mvrnorm(100, mu = c(0, 0), Sigma = sigma)

        plot(
          temp,
          xlab = "x",
          ylab = "y",
          main = paste(
            "Correlation =",
            round(r, 2)
          ),
          xlim = c(-3, 3),
          ylim = c(-3, 3)
        )

        r <- readline("r = ")
        r <- eval(parse(text = r))
      }

      invisible()
    } else {
      stop("Install the package MASS")
    }
  }

#' @describeIn groupedBar Grouped bar chart
#' @export

groupedBar.default <-
  function(
    resp,
    condvar = NULL,
    percent = TRUE,
    print = TRUE,
    cond.name = deparse(substitute(condvar)),
    resp.name = deparse(substitute(resp)),
    ...
  ) {
    cond.name <- cond.name
    resp.name <- resp.name
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
      stop("ggplot2 packaged needed for this function to work.")
    }

    if (!is.factor(resp)) {
      resp <- as.factor(resp)
    } else {
      resp <- droplevels(resp)
    }

    #single variable
    if (is.null(condvar)) {
      #end 1 categorical variable
      m <- length(levels(resp))
      gnames <- levels(resp)

      nmiss <- length(resp) - sum(complete.cases(resp))
      if (nmiss > 0) {
        cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
      }

      if (percent) {
        temp3 <- prop.table(table(resp)) * 100
        ylabel <- "Percent (%)"
      } else {
        temp3 <- table(resp)
        ylabel <- "Count"
      }

      out.df <- as.data.frame(temp3)

      p <- ggplot2::ggplot(out.df) +
        ggplot2::geom_bar(
          ggplot2::aes(x = .data[["resp"]], y = .data[["Freq"]]),
          stat = "identity",
          fill = "steelblue"
        ) +
        ggplot2::labs(x = resp.name, y = ylabel)

      print(p)

      temp4 <- c(as.vector(temp3), sum(temp3))
      names(temp4) <- c(gnames, ylabel)
    } else {
      #Two categorical variables

      if (!is.factor(condvar)) {
        condvar <- as.factor(condvar)
      } else {
        condvar <- droplevels(condvar)
      }

      lenres <- length(resp)
      comCases <- complete.cases(resp, condvar)
      nmiss <- lenres - sum(comCases)

      if (nmiss > 0) {
        cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
      }

      temp2 <- table(condvar, resp)
      n <- dim(temp2)[1] #conditioning variable
      #number of conditioning levels
      m <- dim(temp2)[2]
      condnames <- dimnames(temp2)[[1]]
      respnames <- dimnames(temp2)[[2]]

      if (percent) {
        temp3 <- round(100 * prop.table(table(condvar, resp), 1), 3)
        data <- as.data.frame(temp3)
      } else {
        data <- as.data.frame(temp2)
      }

      p <- ggplot2::ggplot(
        data,
        ggplot2::aes(
          x = .data[["condvar"]],
          y = .data[["Freq"]],
          fill = .data[["resp"]]
        )
      ) +
        ggplot2::geom_bar(position = "dodge", stat = "identity") +
        ggplot2::labs(x = cond.name, y = "Percent", fill = resp.name)

      print(p)

      if (percent) {
        temp4 <- cbind(temp3, rowSums(temp3))
        dimnames(temp4)[[2]][m + 1] <- "Sum (%)"
      } else {
        temp4 <- cbind(temp2, rowSums(temp2))
        dimnames(temp4)[[2]][m + 1] <- "Sum"
      }
    } #end else

    if (print) {
      print(temp4)
    }

    invisible(temp3)
  }

#' Confidence Interval Demonstration
#'
#' Draw many random samples and compute confidence interval.  How many
#' intervals capture the true mean?
#'
#' This simulation will draw 100 random samples from a given population
#' distribution and compute the correpsonding confidence intervals. The 100
#' intervals will be drawn with an indication of the ones that missed the true
#' mean. A histogram of the population will also be created.
#'
#' @param distr distribution of the population to be sampled. Options include
#' \code{"normal"}, \code{"exponential"}, \code{"uniform"} and \code{"binary"}
#' (partial match allowed).
#' @param size sample size
#' @param conf.level confidence level.
#' @return The command invisibly returns the fraction of intervals that capture
#' the true mean.
#' @author Laura Chihara
#' @keywords confidence interval
#' @examples
#'
#' confIntDemo()
#'
#' confIntDemo(distr = "exponential", size = 40)
#'
#'
#' @importFrom stats rbinom rnorm rexp runif qt qnorm qt sd
#' @export
confIntDemo <-
  function(distr = "normal", size = 20, conf.level = .95) {
    if (size > 100000) {
      stop("Choose smaller sample size")
    }
    if (size < 2) {
      stop("Choose larger sample size")
    }

    alpha <- 1 - conf.level
    conf.text <- deparse(substitute(conf.level))
    title.text <- paste(
      "100 confidence intervals, confidence level",
      conf.text,
      sep = " "
    )

    popdist <- pmatch(
      distr,
      c("binary", "normal", "exponential", "uniform"),
      nomatch = NA
    )
    if (is.na(distr)) {
      stop(
        "Distribution must be one of \"binary\", \"normal\", \"exponential\" or \"uniform\""
      )
    }

    pop <- switch(
      popdist,
      binary = rbinom(1000000, 1, .6),
      normal = rnorm(1000000, 10, 2),
      exponential = rexp(100000, rate = 10),
      uniform = runif(1000000, 0, 20)
    )

    #plot population distribution
    if (popdist != 1) {
      hist(pop, xlab = "", main = "")
      title("Population")
    }

    mu <- mean(pop)

    index <- logical(100)
    count <- 0
    temp <- matrix(0, nrow = 100, ncol = 2)
    for (i in 1:100) {
      x <- sample(pop, size, replace = F)
      xbar <- mean(x)
      if (popdist == "binary") {
        se <- sqrt(xbar * (1 - xbar) / size)
        lower <- xbar - abs(qnorm(alpha / 2)) * se
        upper <- xbar + abs(qnorm(alpha / 2)) * se
      } else {
        se <- sd(x) / sqrt(size)
        lower <- xbar - abs(qt(alpha / 2, size - 1)) * se
        upper <- xbar + abs(qt(alpha / 2, size - 1)) * se
      }
      temp[i, 1] <- lower
      temp[i, 2] <- upper

      if (lower < mu && mu < upper) {
        count <- count + 1
        index[i] <- TRUE
      }
    }

    xmin <- min(temp[, 1])
    xmax <- max(temp[, 2])

    #Rstudio doesn't have a dev.new() command
    if (Sys.getenv("RSTUDIO_USER_IDENTITY") == "") {
      grDevices::dev.new()
    }

    plot(seq(xmin, xmax, length = 101), 0:100, type = "n", xlab = "", ylab = "")
    abline(v = mu)

    for (j in 1:100) {
      lines(
        temp[j, ],
        c(j, j),
        col = ifelse(index[j], 1, 2),
        lwd = ifelse(index[j], 1, 2)
      )
    }

    if (popdist == 1) {
      mtext("True proportion", side = 3, at = mu)
      sub.text <- paste(count, " intervals cover true proportion", sep = " ")
      title(main = title.text, sub = sub.text)
    } else {
      mtext("True mu", side = 3, at = mu)
      sub.text <- paste(count, " intervals cover true mean", sep = " ")
      title(main = title.text, sub = sub.text)
    }

    invisible(count / 100)
  }

#' Missing observations in factors
#'
#' In data frames with factor variables, convert any observation with "" into
#' <NA>.
#'
#' In a factor variable with the level \code{""""}, this command will convert
#' this to an \code{<NA>}.
#'
#' @param data a data frame with factor variables.
#' @return Returns the same data frame with \code{""""} replaced by \code{<NA>}
#' in factor variables.
#' @note When importing data from comma separated files (for example), missing
#' values in a categorical variable are often denoted by """. We often do not
#' want to treat this as a level of a factor variable in R.
#' @author Laura Chihara
#' @keywords missing values
#'
#' @export

missingLevel <-
  function(data) {
    if (!is.data.frame(data)) {
      stop("Data must be a data frame")
    }
    m <- dim(data)[2]
    for (i in 1:m) {
      if (is.factor(data[, i])) {
        if (levels(data[, i])[1] == "") {
          is.na(levels(data[, i])[1]) <- TRUE
        }
      }
    }

    return(data)
  }

#' @title Print a summary of an \code{carlboot} object
#'
#' @description
#' Print summary statistics and confidence intervals for an \code{carlboot} object.
#'
#' @param x The carlboot object to print.
#' @param ... not used
#'
#' @rdname print
#' @export
#' @method print carlboot
print.carlboot <- function(x, ...) {
  if (is.null(attr(x, "groups"))) {
    if (attr(x, "statistic") %in% c("correlation", "slope")) {
      cat("\n\t** Bootstrap interval of", attr(x, "statistic"), "\n\n")
      cat(
        " Observed",
        attr(x, "statistic"),
        "between",
        attr(x, "x.name"),
        "and",
        attr(x, "y.name"),
        ":",
        round(attr(x, "observed"), 5),
        "\n"
      )
    } else {
      cat("\n\t** Bootstrap interval for", attr(x, "statistic"), "\n\n")
      cat(
        " Observed ",
        attr(x, "x.name"),
        ":",
        round(attr(x, "observed"), 5),
        "\n"
      )
    }
  } else {
    if (length(attr(x, "groups")) > 1) {
      cat(
        "\n\t** Bootstrap interval for difference of",
        attr(x, "statistic"),
        "\n\n"
      )
      cat(
        " Observed difference of",
        attr(x, "statistic"),
        ":",
        paste(attr(x, "groups"), collapse = " - "),
        "=",
        round(attr(x, "observed"), 5),
        "\n"
      )
    } else {
      cat("\n\t**Bootstrap interval for mean of paired difference \n\n")
      cat(
        " Observed mean of ",
        attr(x, "y.name"),
        "-",
        attr(x, "x.name"),
        ":",
        round(attr(x, "observed"), 5),
        "\n"
      )
    }
  }
  cat(" Mean of bootstrap distribution:", round(mean(x), 5), "\n")
  cat(" Standard error of bootstrap distribution:", round(sd(x), 5), "\n\n")
  cat(" Bootstrap percentile interval\n")
  print(confint(x, level = attr(x, "level")))
  cat("\n\t\t*--------------*\n")

  if (attr(x, "plot.hist")) {
    if (is.null(title)) {
      title <- "Bootstrap distribution"
    }

    phist <- plot(
      x,
      xlab = attr(x, "xlab"),
      ylab = attr(x, "ylab"),
      title = attr(x, "title")
    )
  }

  if (attr(x, "plot.qq")) {
    pqq <- ggplot(data = NULL, aes(sample = as.numeric(x))) +
      geom_qq() +
      geom_qq_line() +
      theme_classic() +
      labs(x = "N(0, 1) Quantiles", y = "Bootstrap Statistics")
  }

  which_plot <- c(phist = attr(x, "plot.hist"), pqq = attr(x, "plot.qq"))

  if (sum(which_plot) == 1) {
    print(get(names(which(which_plot))))
  }

  if (sum(which_plot) == 2) {
    print(patchwork::wrap_plots(phist, pqq, ncol = 2))
  }
}


#' @export
#' @method print carlperm
#' @rdname print
print.carlperm <- function(x, ...) {
  cat("\n\t** Permutation test **\n")
  cat("\n Permutation test with alternative:", attr(x, "alternative"), "\n")
  if (attr(x, "statistic") == "F") {
    cat(" Observed F statistic:", round(attr(x, "observed"), 5), "\n")
    if (is.null(attr(x, "xlab"))) attr(x, "xlab") <- "F statistics"
  } else {
    if (attr(x, "statistic") == "correlation") {
      cat(
        " Observed correlation between",
        attr(x, "x.name"),
        ", ",
        attr(x, "y.name"),
        ":",
        round(attr(x, "observed"), 5),
        "\n"
      )
      if (is.null(attr(x, "xlab"))) attr(x, "xlab") <- "Correlation"
    } else {
      if (attr(x, "statistic") == "slope") {
        cat(
          " Observed slope between",
          attr(x, "x.name"),
          ", ",
          attr(x, "y.name"),
          ": ",
          attr(x, "observed"),
          "\n"
        )
        if (is.null(attr(x, "xlab"))) attr(x, "xlab") <- "Slope"
      } else {
        if (attr(x, "statistic") == "paired difference") {
          cat(" Observed mean\n")
          cat(
            " ",
            attr(x, "x.name"),
            ": ",
            round(attr(x, "var.stats")[1], 5),
            "\t",
            attr(x, "y.name"),
            ": ",
            round(attr(x, "var.stats")[2], 5),
            "\n"
          )
          if (is.null(attr(x, "xlab"))) attr(x, "xlab") <- "Paired differences"
        } else {
          cat(" Observed", attr(x, "statistic"), "\n")
          cat(
            " ",
            attr(x, "groups")[1],
            ": ",
            attr(x, "group.stats")[1],
            "\t",
            attr(x, "groups")[2],
            ": ",
            attr(x, "group.stats")[2],
            "\n"
          )
        }
        cat(" Observed difference:", round(attr(x, "observed"), 5), "\n\n")
      }
    }
  }
  cat(" Mean of permutation distribution:", round(mean(x), 5), "\n")
  cat(" Standard error of permutation distribution:", round(sd(x), 5), "\n")
  cat(" P-value: ", formatC(attr(x, "pval"), digits = 5), "\n")
  cat("\n\t*-------------*\n\n")

  if (attr(x, "plot.hist")) {
    if (is.null(title)) {
      title <- "Permutation distribution"
    }

    phist <- plot(
      x,
      xlab = attr(x, "xlab"),
      ylab = attr(x, "ylab"),
      title = attr(x, "title")
    )
  }

  if (attr(x, "plot.qq")) {
    pqq <- ggplot(data = NULL, aes(sample = as.numeric(x))) +
      geom_qq() +
      geom_qq_line() +
      theme_classic() +
      labs(x = "N(0, 1) Quantiles", y = "Permutation Statistics")
  }

  which_plot <- c(phist = attr(x, "plot.hist"), pqq = attr(x, "plot.qq"))

  if (sum(which_plot) == 1) {
    print(get(names(which(which_plot))))
  }

  if (sum(which_plot) == 2) {
    print(patchwork::wrap_plots(phist, pqq, ncol = 2))
  }
}

#' Bootstrap the correlation
#'
#' Bootstrap the correlation of two numeric variables.
#'
#' Bootstrap the correlation of two numeric variables. The bootstrap mean and
#' standard error are printed as well as a bootstrap percentile confidence
#' interval.
#'
#' Observations with missing values are removed.
#'
#' @aliases bootCor bootCor.default bootCor.formula
#' @param x a numeric vector.
#' @param y a numeric vector.
#' @param conf.level confidence level for the bootstrap ercentile interval.
#' @param B number of times to resample (positive integer greater than 2).
#' @param plot.hist a logical value. If \code{TRUE}, plot the bootstrap
#' distribution of the resampled correlation.
#' @param plot.qq a logical value. If \code{TRUE} a normal quantile-quantile
#' plot of the bootstraped values is created.
#' @param x.name Label for variable x
#' @param y.name Label for variable y
#' @param formula a formula of the form lhs ~ rhs where lhs is a numeric variable giving the data values and rhs a factor with two levels giving the corresponding groups.
#' @param data an optional data frame containing the variables in the formula formula.
#' By default the variables are taken from environment(formula).
#' @param subset an optional vector specifying a subset of observations to be used.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param seed optional argument to \code{\link{set.seed}}
#' @param \dots further arguments to be passed to or from methods.
#' @return The command returns the correlations of the resampled
#' observations.
#' @author Laura Chihara
#' @references Tim Hesterberg's website
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords resampling randomiziation
#' @examples
#'
#'
#' plot(states03$ColGrad, states03$InfMortality)
#' bootCor(InfMortality ~ ColGrad, data = states03, B = 1000)
#' bootCor(states03$ColGrad, states03$InfMortality, B = 1000)
#'
#' @importFrom stats cor
#' @export

bootCor <-
  function(x, ...) {
    UseMethod("bootCor")
  }

#' @describeIn boot Bootstrap a single variable or a grouped variable
#' @export
#' @importFrom patchwork wrap_plots

boot.default <-
  function(
    x,
    group = NULL,
    statistic = mean,
    success = NULL,
    conf.level = 0.95,
    B = 10000,
    plot.hist = TRUE,
    plot.qq = FALSE,
    x.name = deparse(substitute(x)),
    xlab = NULL,
    ylab = NULL,
    title = NULL,
    seed = NULL,
    ...
  ) {
    if (!is.null(seed)) {
      set.seed(seed)
    }

    if (B %% 1 != 0 || B < 2) {
      stop("B must be a positive integer")
    }

    x.name <- x.name  # force before x may be modified below

    stat <- match.fun(statistic)

    is_proportion <- FALSE
    if (!is.numeric(x)) {
      enc <- .encode_binary(x, success)
      x <- enc$x
      is_proportion <- TRUE
    } else if (!is.null(success)) {
      warning("'success' is ignored when x is numeric.")
    }

    #Boot for single numerical variable
    if (is.null(group)) {
      #end if for single numeric variable
      comCases <- complete.cases(x)
      nmiss <- length(x) - sum(comCases)

      if (nmiss > 0) {
        cat(
          "\n\n",
          nmiss,
          "observation(s) removed due to missing values.",
          "\n"
        )
      }

      xc <- x[comCases]
      temp <- numeric(B)

      for (i in 1:B) {
        w <- sample(xc, length(xc), replace = TRUE)
        temp[i] <- stat(w)
      } #end for

      observed <- stat(xc)
      boot.mean <- mean(temp)
    } else {
      #--------------------
      #for two sample bootstrap
      group.name <- deparse(substitute(group))

      if (is.factor(group)) {
        group <- droplevels(group)
      } else {
        group <- as.factor(group)
      }

      if (length(levels(group)) != 2) {
        stop("Grouping variable must have only two levels")
      }

      comCases <- complete.cases(x, group)
      nmiss <- length(x) - sum(comCases)

      if (nmiss > 0) {
        cat(
          "\n\n",
          nmiss,
          "observation(s) removed due to missing values.",
          "\n"
        )
      }

      xc <- x[comCases]
      group <- group[comCases]

      group1 <- xc[group == levels(group)[1]]
      group2 <- xc[group == levels(group)[2]]
      stat1 <- stat(group1)
      stat2 <- stat(group2)
      observed <- stat1 - stat2
      temp <- numeric(B)

      for (i in 1:B) {
        x1 <- sample(group1, length(group1), replace = T)
        x2 <- sample(group2, length(group2), replace = T)
        temp[i] <- stat(x1) - stat(x2)
      } #end for

      boot.mean <- mean(temp)
    } #end else

    class(temp) <- "carlboot"
    attr(temp, "observed") <- observed
    attr(temp, "statistic") <- if (is_proportion) "proportion" else as.character(substitute(statistic))
    attr(temp, "groups") <- levels(group)
    attr(temp, "x.name") <- x.name
    attr(temp, "level") <- conf.level
    attr(temp, "xlab") <- xlab
    attr(temp, "ylab") <- ylab
    attr(temp, "title") <- title
    attr(temp, "plot.hist") <- plot.hist
    attr(temp, "plot.qq") <- plot.qq

    temp
  }

#' Stem and leaf plot
#'
#' Stem and leaf plot. Will accept a factor variable as a second argument to
#' create stem plots for each of the levels.
#'
#' This command is just an enhanced version of R's \code{stem} command. It
#' allows the user to create the stem plot for a numeric variable grouped by
#' the levels of a factor variable.
#'
#' @aliases stemPlot stemPlot.default stemPlot.formula
#' @param x a numeric variable.
#' @param grpvar a factor variable. A stem plot of \code{x} will be created for
#' each level of the factor variable.
#' @param varname name of the numeric variable. This is for printing the output
#' only. Change if you want to print out a name different from the actual
#' variable name.
#' @param grpvarname name of the factor variable.  This is for printing the
#' output only. Change if you want to print out a name different from the
#' actual variable name.
#' @param formula a formula of the form \code{x ~ grpvar} where \code{x} is
#' numeric and \code{grpvar} is a factor variable.
#' @param data a data frame with the variables in the formula.
#' @param subset an optional expression specifying which observations to keep.
#' @param \dots further arguments to be passed to or from methods.
#' @author Laura Chihara
#' @keywords stem plot
#' @examples
#'
#'
#' stemPlot(states03$Births, states03$Region)
#'
#' stemPlot(Births ~ Region, data = states03)
#'
#' @importFrom graphics stem
#' @export

stemPlot <-
  function(x, ...) {
    UseMethod("stemPlot")
  }

#' Grouped bar chart
#'
#' Create a bar chart of a single categorical variable or a grouped bar chart
#' of two categorical variables.
#'
#' For a single factor variable, a bar plot. If two factor variables are given,
#' then a bar plot of \code{x} conditioned by \code{condvar}.  This command
#' uses R's \code{table} command so missing values are automatically removed.
#'
#' @aliases groupedBar groupedBar.default groupedBar.formula
#' @param resp a factor variable. If \code{resp} is numeric, it will be coerced to a
#' factor variable.
#' @param condvar a factor variable to condition on. If \code{NULL}, then a bar
#' plot of just the \code{resp} variable will be created. If \code{condvar} is
#' numeric, it will be coerced to a factor variable.
#' @param percent a logical value. Should the y-axis give percent or counts?
#' @param print a logical value. If \code{TRUE}, print out the table.
#' @param cond.name Label for variable \code{condvar}.
#' @param resp.name Label for variable \code{resp}.
#' @param formula a formula of the form \code{x ~ condvar}. If \code{x} or
#' \code{condvar} is (are) not a factor variable, then it (they) will be
#' coerced into one. Formula can also be \code{ ~ x} for a single factor variable.
#' @param data a data frame that contains the variables in the formula.
#' @param subset an optional vector specifying a subset of observations to be
#' used.
#' @param \dots further arguments to be passed to or from methods.
#' @return Returns invisibly a table of the variable(s).
#' @author Laura Chihara
#' @keywords bar plot grouped bar plot
#' @examples
#'
#'
#' groupedBar(states03$Region)
#'
#' \dontrun{
#' groupedBar(states03$DeathPenalty, states03$Region, legend.loc = "topleft")
#'
#' #Using a formula syntax:
#'
#' groupedBar(~Region, data = states03)
#' groupedBar(DeathPenalty ~ Region, data = states03, legend.loc = "topleft")
#' }
#'
#' @export

groupedBar <-
  function(resp, ...) {
    UseMethod("groupedBar")
  }

.calc_pvalue <- function(perms, observed, alt) {
  B <- length(perms)

  switch(
    alt,
    (sum(perms <= observed) + 1) / (B + 1),
    (sum(perms >= observed) + 1) / (B + 1),
    (sum(abs(perms) >= abs(observed)) + 1) / (B + 1),
    stop("Alternative not matched.")
  )
}

#' Anova F test
#'
#' ANOVA F test when given summarized data (sample sizes, means and standard
#' deviations).
#'
#' Perform an ANOVA F test when presented with summarized data: sample sizes,
#' sample means and sample standard devations.
#'
#' @param N a vector with the sample sizes
#' @param mn a vector of means, one for each group in the sample
#' @param stdev a vector of standard deviations, one for each group in the
#' sample
#' @return Returns invisibly a list
#'
#' \item{Treatment SS}{The treatment sum of squares (also called the "between
#' sum of squares").} \item{Residual SS}{Residual sum of squares (also called
#' the "within sum of squares").} \item{Degrees of Freedom}{a vector with the
#' numerator and denominator degrees of freedom.} \item{Treatment Mean
#' Square}{Treatment SS/numerator DF} \item{Residual Mean Square}{Residual
#' SS/denominator DF} \item{Residual Standard Error}{Square root of Residual
#' Mean Square} \item{F}{the F statistic} \item{P-value}{p-value} ...
#' @author Laura Chihara
#' @keywords ANOVA Summarized data
#' @examples
#'
#' #use the data set chickwts from base R
#' head(chickwts)
#'
#' N <- table(chickwts$feed)
#' stdev <- tapply(chickwts$weight, chickwts$feed, sd)
#' mn <- tapply(chickwts$weight, chickwts$feed, mean)
#'
#' anovaSummarized(N, mn, stdev)
#'
#' @importFrom stats pf
#' @export

anovaSummarized <-
  function(N, mn, stdev) {
    ##First argument must be vector of non-negative integers
    if (any(N < 0)) {
      stop("First argument must be vector of positive integers.")
    }
    if (length(N) != length(mn) || length(N) != length(stdev)) {
      stop("All arguments must be vectors of equal length.")
    }

    TrSS <- sum(N * mn^2) - (sum(N * mn))^2 / sum(N)
    RSS <- sum((N - 1) * stdev^2)
    df1 <- length(N) - 1
    TrMS <- TrSS / df1
    df2 <- sum(N) - length(N)
    RMS <- RSS / df2
    my.f <- TrMS / RMS
    p <- 1 - pf(my.f, df1, df2)
    resSE <- sqrt(RSS / df2)

    out <- data.frame(c(TrSS, RSS, df1, df2, resSE))
    names(out) <- ""
    row.names(out) <- c(
      "Treatment SS",
      "Residual SS",
      "numerator DF",
      "denominator DF",
      "Residual standard error"
    )

    stat <- c("F-stat" = my.f, "P-value" = p)

    print(out)
    cat("\n")
    print(stat, justify = "left")

    my.list <- list(
      "Treatment SS" = TrSS,
      "Residual SS" = RSS,
      "Degrees of Freedom" = c(df1, df2),
      "Treatment Mean Square" = TrMS,
      "Residual Mean Square" = RMS,
      "Residual Standard Error" = resSE,
      "F" = my.f,
      "P-value" = p
    )
    invisible(my.list)
  }

#' Permutation test for paired data.
#'
#' Permutation test for paired data.
#'
#' For two paired numeric variables with n rows, randomly select k of the n
#' rows (k also is randm) and switch the entries \eqn{latex}{(a, b) \to (b, a)}
#' and then compute the mean of the difference of the two variables (\code{y-x}).
#'
#' Observations with missing values are removed.
#'
#' @aliases permTestPaired permTestPaired.default permTestPaired.formula
#' @param x a numeric vector.
#' @param y a numeric vector.
#' @param B the number of resamples.
#' @param seed optional argument to \code{\link{set.seed}}
#' @param alternative the alternative hypothesis. Options are
#' \code{"two.sided"}, \code{"less"} and \code{"greater"}.
#' @param plot.hist a logical value. If \code{TRUE}, create a histogram
#' displaying the permutation distribution of the statistic.
#' @param plot.qq a logical value. If \code{TRUE}, include a quantile-normal
#' plot of the permuation distribution.
#' @param x.name Label for x variable
#' @param y.name Label for y variable
#' @param formula a formula of the form \code{y ~ x}, where \code{x, y} are both numeric variables.
#' @param data an optional data frame containing the variables in the formula. By default the
#' variables are taken from environment(formula).
#' @param subset	an optional vector specifying a subset of observations to be used.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param \dots further arguments to be passed to or from methods.
#' @return Returns invisibly a vector of the replicates of the test statistic
#' (ex. mean of the difference of the resampled variables).
#' @author Laura Chihara
#' @references Tim Hesterberg's website:
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords permutation test randomization resampling paired data
#' @examples
#'
#' #Does chocolate ice cream have more calories than vanilla ice cream, on average?
#' #H0: mean number of calories is the same
#' #HA: mean number of calories is greater in chocolate ice cream
#'
#' permTestPaired(Icecream$VanillaCalories, Icecream$ChocCalories, alternative = "less")
#' permTestPaired(ChocCalories ~ VanillaCalories, data = Icecream, alternative = "greater")
#'
#' @export

permTestPaired <-
  function(x, ...) {
    UseMethod("permTestPaired")
  }

#' @describeIn bootCor Bootstrap the correlation of two numeric variables.
#' @export

bootCor.default <-
  function(
    x,
    y,
    conf.level = 0.95,
    B = 10000,
    plot.hist = TRUE,
    xlab = NULL,
    ylab = NULL,
    title = NULL,
    plot.qq = FALSE,
    x.name = deparse(substitute(x)),
    y.name = deparse(substitute(y)),
    seed = NULL,
    ...
  ) {
    if (B %% 1 != 0 || B < 2) {
      stop("B must be a positive integer")
    }

    if (!is.null(seed)) {
      set.seed(seed)
    }

    if (!is.numeric(x) || !is.numeric(y)) {
      stop("Both variables must be numeric.")
    }

    comCases <- complete.cases(x, y)
    nmiss <- length(x) - sum(comCases)
    if (nmiss > 0) {
      cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
    }

    x2 <- x[comCases]
    y2 <- y[comCases]

    observed <- cor(x2, y2)
    n <- length(x2)
    #Boot
    temp <- numeric(B)
    for (i in 1:B) {
      index <- sample(1:n, n, replace = TRUE)
      newx <- x2[index]
      newy <- y2[index]
      temp[i] <- cor(newx, newy)
    } #end for
    class(temp) <- "carlboot"
    attr(temp, "observed") <- observed
    attr(temp, "statistic") <- "correlation"
    attr(temp, "groups") <- NULL
    attr(temp, "x.name") <- x.name
    attr(temp, "y.name") <- y.name
    attr(temp, "level") <- conf.level
    attr(temp, "xlab") <- xlab
    attr(temp, "ylab") <- ylab
    attr(temp, "title") <- title
    attr(temp, "plot.hist") <- plot.hist
    attr(temp, "plot.qq") <- plot.qq

    temp
  }

#' Demonstration of the normal qq-plot.
#'
#' Demonstrate the normal quantile-quantile plot for samples drawn from
#' different populations.
#'
#' Draw a random sample from the chosen sample and display the normal qq-plot
#' as well as the histogram of its distribution.
#'
#' @param n sample size
#' @param distribution population distribution. Options are \code{"normal"},
#' \code{"t"},\code{"exponential"}, \code{"chi.square"}, \code{"F"} or
#' \code{"beta"} (partial matches are accepted).
#' @param mu mean for the normal distribution.
#' @param sigma (positive) standard deviation for the normal distribution.
#' @param df (positive) degrees of freedom for the t-distribution.
#' @param lambda positive rate for the exponential distribution.
#' @param numdf (positive) numerator degrees of freedom for the chi-square
#' distribution.
#' @param dendf (positive) denominator degrees of freedom for the chi-square
#' distribution.
#' @param shape1 positive parameter for the beta distribution (shape1 = a).
#' @param shape2 positive parameter for the beta distribution (shape2 = b).
#' @return Returns invisibly the random sample.
#' @author Laura Chihara
#' @keywords normal quantile-quantile plot
#' @examples
#'
#' qqPlotDemo(n = 30, distr = "exponential", lambda = 1/3)
#'
#'
#' @importFrom stats rchisq rnorm rt rf rbeta qqnorm qqline
#' @export

qqPlotDemo <-
  function(
    n = 25,
    distribution = "normal",
    mu = 0,
    sigma = 1,
    df = 10,
    lambda = 10,
    numdf = 10,
    dendf = 16,
    shape1 = 40,
    shape2 = 5
  ) {
    #get random sample from a distribution and plot its
    #histogram and normal quantile-quantile plot

    distr <- pmatch(
      distribution,
      c("normal", "t", "exponential", "chi.square", "F", "beta"),
      nomatch = NA
    )
    if (is.na(distr)) {
      stop(
        "Distribution must be one of \"normal\", \"t\", \"exponential\", \"chi.square\", \"F\", or \"beta\" "
      )
    }

    if (
      sigma <= 0 ||
        df <= 0 ||
        lambda <= 0 ||
        numdf <= 0 ||
        dendf <= 0 ||
        shape1 <= 0 ||
        shape2 <= 0
    ) {
      stop("Parameter must be positive.")
    }

    x <- switch(
      distr,
      normal = rnorm(n, mu, sigma),
      t = rt(n, df),
      exponential = rexp(n, rate = lambda),
      chi.square = rchisq(n, df),
      F = rf(n, numdf, dendf),
      beta = rbeta(n, shape1, shape2)
    )

    distr.expand <- char.expand(
      distribution,
      c("normal", "t", "exponential", "chi.square", "F", "beta"),
      nomatch = warning("No match")
    )

    par.old <- par(
      mfrow = c(2, 1),
      mar = c(2.1, 4.1, 2, 2),
      cex.main = .8,
      cex.axis = .8,
      cex.lab = .8
    )

    hist(x, main = "", xlab = "")
    title(paste(
      "Sample size ",
      n,
      "; ",
      distr.expand,
      "distribution",
      sep = " "
    ))
    par(mar = c(4.1, 4.1, 2, 3))
    qqnorm(x)
    qqline(x)

    on.exit(par(par.old))
    invisible(x)
  }

#' Bootstrap
#'
#' Bootstrap a single variable or a grouped variable
#'
#' Perform a bootstrap of a statistic applied to a single variable, or to the
#' difference of the statistic computed on two samples (using the grouping
#' variable). If \code{x} is a binary vector of 0's and 1's and the function is
#' the mean, then the statistic of interest is the proportion.
#'
#' Observations with missing values are removed.
#'
#' @aliases boot boot.default boot.formula
#' @param x a numeric, logical, factor, or character vector. Logical, factor,
#' and character vectors with exactly two unique values are converted to 0/1,
#' and \code{mean} is used to compute the proportion.
#' @param group an optional grouping variable (vector), usually a factor
#' variable. If it is a binary numeric variable, it will be coerced to a
#' factor.
#' @param statistic function that computes the statistic of interest. Default
#' is the \code{mean}.
#' @param success a character string naming the level of \code{x} to code as
#' 1 when \code{x} is a logical, factor, or character variable. Defaults to
#' \code{NULL}, which uses the second factor level (alphabetically) or
#' \code{TRUE} for logical vectors.
#' @param conf.level confidence level for the bootstrap percentile interval.
#' Default is 95\%.
#' @param B number of times to resample (positive integer greater than 2).
#' @param seed optional argument to \code{\link{set.seed}}
#' @param plot.hist logical value. If \code{TRUE}, plot the histogram of the
#' bootstrap distribution.
#' @param plot.qq Logical value. If \code{TRUE}, create a normal
#' quantile-quantile plot of the bootstrap distribution.
#' @param x.name Label for variable name
#' @param formula a formula \code{y ~ g} where \code{y} is a numeric vector and
#' \code{g} a factor variable with two levels. If \code{g} is a binary numeric
#' vector, it will be coerced to a factor variable. For a single numeric variable,
#' formula may also be \code{~ y}.
#' @param data a data frame that contains the variables given in the formula.
#' @param subset an optional expression indicating what observations to use.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param \dots further arguments to be passed to or from methods.
#' @return A vector with the resampled statistics is returned invisibly.
#' @author Laura Chihara
#' @references Tim Hesterberg's website
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords bootstrap resampling randomization
#' @examples
#'
#' #ToothGrowth data (supplied by R)
#' #bootstrap mean of a single numeric variable
#' boot(ToothGrowth$len)
#'
#' #bootstrap difference in mean of tooth length for two groups.
#' boot(ToothGrowth$len, ToothGrowth$supp, B = 1000)
#'
#' #same as above using formula syntax
#' boot(len ~ supp, data = ToothGrowth, B = 1000)
#'
#' # Penguin Survival proportion
#' boot(penguin_survival$Status, data = penguin_survival, B = 1000)
#'
#' # same as above, but with the formula syntax
#' boot(~Status, data = penguin_survival, B = 1000)
#'
#' # Penguin Survival if tagged vs. untagged
#' # bootstrap difference in proportions of survival
#' boot(penguin_survival$Status, penguin_survival$TagType, B = 1000)
#'
#' # now using the formula syntax
#' boot(Status ~ TagType, data = penguin_survival, B = 1000)
#'
#' @importFrom stats complete.cases sd terms quantile
#' @importFrom graphics abline hist legend lines mtext par plot points title
#' @importFrom grDevices dev.new
#' @export

boot <-
  function(x, ...) {
    UseMethod("boot")
  }

#' @describeIn permTestAnova Permutation test for ANOVA F-test
#' @export

permTestAnova.formula <-
  function(formula, data = parent.frame(), subset, ...) {
    if (
      missing(formula) ||
        (length(formula) != 3L) ||
        (length(attr(terms(formula[-2L]), "term.labels")) != 1L)
    ) {
      stop("'formula' missing or incorrect")
    }
    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, parent.frame()))) {
      m$data <- as.data.frame(data)
    }
    m[[1L]] <- as.name("model.frame")
    m$... <- NULL
    mf <- eval(m, parent.frame())

    nmiss <- length(attr(mf, "na.action"))
    if (nmiss > 0) {
      cat("\n ", nmiss, "observations removed due to missing values.\n")
    }

    response <- attr(attr(mf, "terms"), "response")
    x <- mf[[response]]
    g <- mf[[-response]]

    y <- do.call("permTestAnova", c(list(x, g), ...))

    y
  }

#' @describeIn permTestAnova Permutation test for ANOVA F-test
#' @importFrom stats aov
#' @export

permTestAnova.default <- function(
  x,
  group,
  B = 9999,
  plot.hist = TRUE,
  plot.qq = FALSE,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
) {
  if (!is.null(seed)) {
    set.seed(seed)
  }

  if (!is.numeric(x)) {
    stop("Variable must be numeric.")
  }
  if (is.factor(group)) {
    group <- droplevels(group)
  } else {
    group <- as.factor(group)
  }

  comCases <- complete.cases(x, group)
  nmiss <- length(x) - sum(comCases)
  if (nmiss > 0) {
    cat("\n ", nmiss, "observations removed due to missing values.\n")
  }

  x <- x[comCases]
  group <- group[comCases]

  n <- length(x)
  observed <- summary(aov(x ~ group))[[1]][["F value"]][1]

  result <- numeric(B)
  for (i in 1:B) {
    index <- sample(n, n, replace = FALSE)
    newgroup <- group[index]
    result[i] <- summary(aov(x ~ newgroup))[[1]][["F value"]][1]
  } #end for

  P.value <- .calc_pvalue(result, observed, alt = 2)

  if (P.value > 1) {
    P.value <- 1
  }

  df1 <- length(unique(group)) - 1

  class(result) <- "carlperm"
  attr(result, "observed") <- observed
  attr(result, "statistic") <- "F"
  attr(result, "df") <- c(df1, n - df1 - 1)
  attr(result, "groups") <- levels(group)
  attr(result, "alternative") <- "greater"
  attr(result, "pval") <- P.value
  attr(result, "xlab") <- xlab
  attr(result, "ylab") <- ylab
  attr(result, "title") <- title
  attr(result, "plot.hist") <- plot.hist
  attr(result, "plot.qq") <- plot.qq

  result
}

#' Ice cream data
#'
#' Nutritional information on vanilla and chocolate ice cream from a sample of
#' companies.
#'
#' @name Icecream
#' @docType data
#' @format A data frame with 39 observations on the following 7 variables.
#' \describe{
#' \item{Brand}{Brand name}
#' \item{VanillaCalories}{Calories per serving in vanilla}
#' \item{VanillaFat}{Fat per serving (g) in vanilla}
#' \item{VanillaSugar}{Sugar per serving (g) in vanilla}
#' \item{ChocCalories}{Calories per serving in chocolate}
#' \item{ChocFat}{Fat per serving (g) in chocolate}
#' \item{ChocSugar}{Sugar per serving (g) in chocolate}
#' }
#' @source Data collected by Carleton student Ann Butkowski (2008).
#' @keywords datasets
#' @examples
#'
#' head(Icecream)
#' t.test(Icecream$VanillaCalories, Icecream$ChocCalories, paired = TRUE)
#'
NULL


#' Milkshakes (chocolate)
#' Nutrional information on chocolate milkshakes from a sample of restaurants.
#'
#' @name Milkshakes
#' @docType data
#' @format A data frame with 29 observations on the following 11 variables.
#' \describe{
#' \item{Restaurant}{Names of restaurants}
#' \item{Type}{Type of restaurant, \code{Dine In} \code{Fast Food}}
#' \item{Calories}{Calories per serving}
#' \item{Fat}{Fat per
#' serving (g)}
#' \item{Sodium}{Sodium per serving (mg)}
#' \item{Carbs}{Carbohydrates per serving (g)}
#' \item{SizeOunces}{Size of milkshake (ounces)}
#' \item{CalPerOunce}{Calories per ounce}
#' \item{FatPerOunce}{Fat per ounce}
#' \item{CarbsPerOunce}{Carbohydrates per ounce}
#' }
#' @source Data collected by Carleton students Yoni Blumberg (2013) and Lindsay
#' Guthrie (2013).
#' @keywords datasets
NULL

#' Penguin survival by tag type
#'
#' Survival outcomes for penguins fitted with metal or electronic tags.
#'
#' @name penguin_survival
#' @docType data
#' @format A data frame with 2 observations on the following 3 variables.
#' \describe{
#'   \item{TagType}{Tag type: \code{Metal} or \code{Electronic}}
#'   \item{Status}{Survival status: \code{Survived} or \code{Electronic}}
#' }
#' @source Saraux, C., et al., “Reliability of flipper-banded penguins as
#' indicators of climate change,” Nature, January 2011; 469: 203–206.
#'
#' Lock, Robin H., et al. Statistics: Unlocking the power of data.
#' John Wiley & Sons, 2025.
#'
#' @keywords datasets
NULL

#' US government data, 2003
#'
#' Census data on the 50 states from 2003.
#'
#' @name states03
#' @docType data
#' @format A data frame with 50 observations on the following 24 variables.
#' \describe{
#' \item{State}{the 50 states}
#' \item{Region}{a
#' factor with levels \code{Midwest}, \code{Northeast}, \code{South},
#' \code{West}}
#' \item{Pop}{Population in 1000}
#' \item{Births}{Number of births}
#' \item{Deaths}{Number of
#' deaths}
#' \item{Pop18}{Percent of population 18 years of age or
#' younger}
#' \item{Pop65}{Percent of population 65 years of age or
#' older}
#' \item{HSGrad}{Percent of population 25 years of age or older
#' with a high school degree}
#' \item{ColGrad}{Percent of population 25
#' years of age or older with a college degree}
#' \item{TeacherPay}{Average teachers salary in dollars}
#' \item{InfMortality}{Infant mortality per 1000 live births}
#' \item{TeenBirths}{Live births per 1000 15-19 year old females}
#' \item{ViolentCrime}{Violent crime per 100000 population}
#' \item{PropertyCrime}{Property crime per 100000 population}
#' \item{DeathPenalty}{State has death penalty?}
#' \item{Executions}{Number of executions 1977-2003}
#' \item{Poverty}{Percent of populaton below the poverty level}
#' \item{Unemp}{Percent unemployed (of population 16 years or older)}
#' \item{Uninsured}{Percent uninsured (3 year aveage)}
#' \item{Income}{Median household income in 1998 dollars}
#' \item{Earnings}{Average hourly earnings of production workers in
#' manufacturing}
#' \item{Heart}{Deaths by heart disease per 100000
#' population}
#' \item{Vehicles}{Deaths by motor vehicle accidents per
#' 100000 population}
#'  \item{Homeowners}{Home ownership rate}
#' }
#' @source United States Census Bureau \url{https://www.census.gov/}
#' @keywords datasets
NULL

#' @describeIn stemPlot Stem and leaf plot
#' @export

stemPlot.default <-
  function(x, grpvar = NULL, varname = NULL, grpvarname = NULL, ...) {
    if (!is.numeric(x)) {
      stop("variable must be numeric")
    }

    if (is.null(varname)) {
      varname <- deparse(substitute(x))
    }

    comCases <- complete.cases(x)
    nmiss <- length(x) - sum(comCases)
    if (nmiss > 0) {
      cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
    }

    cat("\n***Stem and Leaf plot for ", varname, "***\n")

    if (!is.null(grpvar)) {
      if (is.null(grpvarname)) {
        grpvarname <- deparse(substitute(grpvar))
      }

      if (!is.factor(grpvar)) {
        grpvar <- as.factor(grpvar)
      } else {
        grpvar <- droplevels(grpvar)
      }

      grp.levels <- levels(grpvar)

      cat("   Grouped by levels of ", grpvarname, "\n")
      for (i in grp.levels) {
        cat("\n   ", i, "\n :")
        stem(x[grpvar == i], ...)
      } #end for
    } else {
      stem(x, ...)
    }

    invisible()
  }

#' Permutation test for ANOVA F-test
#'
#' Permutation test to see if the population mean is the same for two or more
#' populations. For instance, test \eqn{latex}{H_0: \mu_1 = \mu_2 = \mu_3} where
#' \eqn{latex}{\mu} denotes the population mean. The values of the numeric
#' variable are randomly assigned to the groups and the ANOVA F statistic is
#' calculated. The command will print the mean and
#' standard error of the distribution of the test statistic as well as a
#' P-value.
#'
#' Observations with missing values are removed.
#'
#' @aliases permTestAnova permTestAnova.default permTestAnova.formula
#' @param x a numeric vector.
#' @param group a factor variable with two or more levels. If \code{group} is a
#' numeric vector, it will be coerced into a factor variable.
#' @param B the number of resamples (positive integer greater than 2).
#' @param seed optional argument to \code{\link{set.seed}}
#' @param plot.hist a logical value. If \code{TRUE}, the permutation
#' distribution of the statistic is plotted.
#' @param plot.qq a logical value. If \code{TRUE}, then a normal
#' quantile-quantile plot of the resampled test statistic is created.
#' @param formula a formula of the form \code{y ~ group} where \code{y} is
#' numeric and \code{group} is a factor variable.
#' @param data a data frame with the variables in the formula.
#' @param subset an optional expression specifying which observations to keep.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param \dots further arguments to be passed to or from methods.
#' @return Returns invisibly a vector of the replicates of the test statistic.
#' @author Adam Loy, Laura Chihara
#' @references Tim Hesteberg's website:
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords permutation test resampling randomization
#' @examples
#'
#' permTestAnova(states03$ViolentCrime, states03$Region, B = 499)
#'
#' #using formula syntax
#' \dontrun{
#' permTestAnova(ViolentCrime ~ Region, data = states03, B = 9999)
#' }
#'
#' @export

permTestAnova <-
  function(x, ...) {
    UseMethod("permTestAnova")
  }

#' @describeIn boot Bootstrap a single variable or a grouped variable
#' @export

boot.formula <-
  function(formula, data, subset, ...) {
    if (missing(formula) || (length(formula) > 3L)) {
      stop("'formula' missing or incorrect")
    }

    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, parent.frame()))) {
      m$data <- as.data.frame(data)
    }
    m[[1L]] <- as.name("model.frame")
    m$... <- NULL
    mf <- eval(m, parent.frame())

    nmiss <- length(attr(mf, "na.action"))
    if (nmiss > 0) {
      cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
    }
    varnames <- names(mf)

    if (length(formula) == 2L) {
      g <- NULL
      y <- mf[[1]]
    } else {
      response <- attr(attr(mf, "terms"), "response")
      y <- mf[[response]]
      g <- mf[[-response]]
    }

    y <- do.call("boot", c(list(y, g, x.name = varnames[1]), list(...)))

    y
  }

#' Bootstrap paired data
#'
#' Perform a bootstrap of two paired variables.
#'
#' The command will compute the difference of \code{x} and \code{y} and
#' bootstrap the difference. The mean and standard error of the bootstrap
#' distribution will be printed as well as a bootstrap percentile interval.
#'
#' Observations with missing values are removed.
#'
#' @aliases bootPaired bootPaired.default bootPaired.formula
#' @param x a numeric vector.
#' @param y a numeric vector.
#' @param conf.level confidence level for the bootstrap percentile interval.
#' @param B number of resamples (positive integer greater than 2).
#' @param plot.hist logical. If \code{TRUE}, plot the histogram of the
#' bootstrap distribution.
#' @param plot.qq logical. If \code{TRUE}, a normal quantile-quantile plot of
#' the replicates will be created.
#' @param x.name Label for variable x
#' @param y.name Label for variable y
#' @param formula a formula \code{y ~ x} where \code{x, y} are both numeric vectors
#' @param data a data frame that contains the variables given in the formula.
#' @param subset an optional expression indicating what observations to use.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param seed optional argument to \code{\link{set.seed}}
#' @param \dots further arguments to be passed to or from methods.
#' @return The command returns a vector with the replicates of the
#' statistic being bootstrapped.
#' @author Laura Chihara
#' @references Tim Hesterberg's website
#' \url{https://www.timhesterberg.net/bootstrap-and-resampling}
#' @keywords bootstrap randomization resampling
#' @examples
#'
#' #Bootstrap the mean difference of fat content in vanilla and chocolate ice
#' #cream. Data are paired becaues ice cream from the same manufacturer will
#' #have similar content.
#' Icecream
#' bootPaired(ChocFat ~ VanillaFat, data = Icecream)
#' bootPaired(Icecream$VanillaFat, Icecream$ChocFat)
#'
#' @export

bootPaired <-
  function(x, ...) {
    UseMethod("bootPaired")
  }

#' @title Plot the bootstrap distribution in \code{carlboot} object
#'
#' @description
#' Plot the bootstrap distribution returned as a \code{carlboot} object.
#'
#' @param x The carlboot object to print.
#' @param bins number of bins in histogram.
#' @param size size of points.
#' @param xlab an optional character string for the x-axis label
#' @param ylab an optional character string for the y-axis label
#' @param title an optional character string giving the plot title
#' @param ... not used
#'
#' @rdname plot
#' @export
#' @method plot carlboot
#' @importFrom scales label_percent
#' @importFrom stats density
#' @examples
#'
#' boot_dist <- boot(ToothGrowth$len, ToothGrowth$supp, B = 1000)
#' plot(boot_dist)
plot.carlboot <- function(
  x,
  bins = 15,
  size = 5,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  ...
) {
  boot_stats <- as.numeric(x)
  width <- NULL
  p <- ggplot2::ggplot(data = NULL, ggplot2::aes(x = boot_stats)) +
    ggplot2::geom_histogram(aes(y = after_stat(width * density)), bins = bins) +
    ggplot2::geom_point(
      aes(x = mean(x), y = 0, color = "Bootstrap", shape = "Bootstrap"),
      size = size
    ) +
    ggplot2::geom_point(
      aes(
        x = attr(x, "observed"),
        color = "Observed",
        shape = "Observed",
        y = 0
      ),
      size = size
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::label_percent(),
      expand = expansion(mult = c(0.05, .05))
    ) +
    ggplot2::labs(
      title = "Bootstrap distribution",
      x = "statistics",
      y = "Percentage",
      shape = "",
      color = ""
    ) +
    ggplot2::scale_shape_manual(values = c(16, 4)) +
    ggplot2::scale_color_manual(values = c("darkorange", "skyblue")) +
    ggplot2::theme_classic()

  if (!is.null(ylab)) {
    p <- p + ggplot2::ylab(ylab)
  }

  if (!is.null(xlab)) {
    p <- p + ggplot2::xlab(xlab)
  }

  if (!is.null(title)) {
    p <- p + ggplot2::labs(title = title)
  }

  p
}


#' @export
#' @method plot carlperm
#' @rdname plot
#' @examples
#'
#' perm_dist <- permTest(states03$ViolentCrime, states03$DeathPenalty, B = 999)
#' plot(perm_dist)
plot.carlperm <- function(
  x,
  bins = 15,
  size = 5,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  ...
) {
  boot_stats <- as.numeric(x)
  width <- NULL
  p <- ggplot2::ggplot(data = NULL, ggplot2::aes(x = boot_stats)) +
    ggplot2::geom_histogram(aes(y = after_stat(width * density)), bins = bins) +
    ggplot2::geom_point(
      aes(x = mean(x), y = 0, color = "Permutation", shape = "Permutation"),
      size = size
    ) +
    ggplot2::geom_point(
      aes(
        x = attr(x, "observed"),
        color = "Observed",
        shape = "Observed",
        y = 0
      ),
      size = size
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::label_percent(),
      expand = expansion(mult = c(0.05, .05))
    ) +
    ggplot2::labs(
      title = "Permutation distribution",
      x = "Statistics",
      y = "Percentage",
      shape = "",
      color = ""
    ) +
    ggplot2::scale_shape_manual(values = c(16, 4)) +
    ggplot2::scale_color_manual(values = c("darkorange", "skyblue")) +
    ggplot2::theme_classic()

  if (!is.null(ylab)) {
    p <- p + ggplot2::ylab(ylab)
  }

  if (!is.null(xlab)) {
    p <- p + ggplot2::xlab(xlab)
  }

  if (!is.null(title)) {
    p <- p + ggplot2::labs(title = title)
  }

  p
}

#' @title Calculate a CI from a \code{carlboot} object
#'
#' @description
#' Calculate percentile confidence intervals for a \code{carlboot} object.
#'
#' @param object The carlboot object to print.
#' @param parm not used in CarletonStats, just for generic consistency
#' @param level the confidence level
#' @param ... not used
#'
#' @rdname confint
#' @export
#' @method confint carlboot
#' @importFrom stats confint
#' @import ggplot2
confint.carlboot <- function(object, parm = NULL, level = 0.95, ...) {
  quantile(object, probs = (1 + c(-level, level)) / 2)
}

#' @describeIn permTestCor Permutation test for the correlation of two variables.
#' @export

permTestCor.default <-
  function(
    x,
    y,
    B = 999,
    alternative = "two.sided",
    plot.hist = TRUE,
    plot.qq = FALSE,
    x.name = deparse(substitute(x)),
    y.name = deparse(substitute(y)),
    xlab = NULL,
    ylab = NULL,
    title = NULL,
    seed = NULL,
    ...
  ) {
    if (!is.null(seed)) {
      set.seed(seed)
    }

    if (!is.numeric(x)) {
      stop("Variable 1 must be numeric")
    }
    if (!is.numeric(y)) {
      stop("Variable 2 must be numeric")
    }

    comCases <- complete.cases(x, y)
    nmiss <- length(x) - sum(comCases)
    if (nmiss > 0) {
      cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
    }

    x2 <- x[comCases]
    y2 <- y[comCases]

    n <- length(x2)
    observed <- round(cor(x2, y2), 4)

    result <- numeric(B)
    for (i in 1:B) {
      index <- sample(n, n, replace = FALSE)
      newy <- y2[index]
      result[i] <- cor(x2, newy)
    } #end for

    alt <- pmatch(alternative, c("less", "greater", "two.sided"), nomatch = 4)

    P.value <- .calc_pvalue(result, observed, alt)
    if (P.value > 1) {
      P.value <- 1
    }

    class(result) <- "carlperm"
    attr(result, "observed") <- observed
    attr(result, "statistic") <- "correlation"
    attr(result, "alternative") <- alternative
    attr(result, "x.name") <- x.name
    attr(result, "y.name") <- y.name
    attr(result, "pval") <- P.value
    attr(result, "xlab") <- xlab
    attr(result, "ylab") <- ylab
    attr(result, "title") <- title
    attr(result, "plot.hist") <- plot.hist
    attr(result, "plot.qq") <- plot.qq

    result
  }

#' @describeIn bootSlope Bootstrap the slope of a simple linear regression line
#' @export

bootSlope.formula <-
  function(formula, data, subset, ...) {
    if (
      missing(formula) ||
        (length(formula) != 3L) ||
        (length(attr(terms(formula[-2L]), "term.labels")) != 1L)
    ) {
      stop("'formula' missing or incorrect")
    }

    parsed <- .parse_formula_two_var(
      match.call(expand.dots = FALSE),
      parent.frame()
    )
    do.call(
      "bootSlope",
      c(
        list(
          parsed$x,
          parsed$y,
          x.name = parsed$x.name,
          y.name = parsed$y.name
        ),
        list(...)
      )
    )
  }

#' @describeIn groupedBar Grouped bar chart
#' @export

groupedBar.formula <-
  function(formula, data = parent.frame(), subset, ...) {
    if (length(formula) > 3L) {
      stop("'formula' incorrect")
    }

    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, parent.frame()))) {
      m$data <- as.data.frame(data)
    }
    m[[1L]] <- as.name("model.frame")
    m$... <- NULL
    mf <- eval(m, parent.frame())
    varnames <- names(mf)

    nmiss <- length(attr(mf, "na.action"))
    if (nmiss > 0) {
      cat("\n ", nmiss, "observation(s) removed due to missing values.\n")
    }

    if (length(formula) == 2L) {
      do.call(
        "groupedBar",
        c(list(resp = unlist(mf), resp.name = varnames[1]), list(...))
      )
    } else {
      response <- attr(attr(mf, "terms"), "response")
      y <- mf[[response]]
      g <- mf[[-response]]
      names(g) <- varnames[2]

      y <- do.call(
        "groupedBar",
        c(
          list(y, g, cond.name = varnames[2], resp.name = varnames[1]),
          list(...)
        )
      )
    } #end else
  }

# boot() print output is stable

    Code
      print(result)
    Output
      
      	** Bootstrap interval for mean 
      
       Observed  ToothGrowth$len : 18.81333 
       Mean of bootstrap distribution: 18.83433 
       Standard error of bootstrap distribution: 0.9604 
      
       Bootstrap percentile interval
          2.5%    97.5% 
      16.87433 20.63175 
      
      		*--------------*

# boot() two-group print output is stable

    Code
      print(result)
    Output
      
      	** Bootstrap interval for difference of mean 
      
       Observed difference of mean : OJ - VC = 3.7 
       Mean of bootstrap distribution: 3.65638 
       Standard error of bootstrap distribution: 1.84858 
      
       Bootstrap percentile interval
           2.5%     97.5% 
      -0.113500  7.187833 
      
      		*--------------*

# bootPaired() print output is stable

    Code
      print(result)
    Output
      
      	**Bootstrap interval for mean of paired difference 
      
       Observed mean of  Icecream$ChocCalories - Icecream$VanillaCalories : 7.33333 
       Mean of bootstrap distribution: 7.44503 
       Standard error of bootstrap distribution: 2.01653 
      
       Bootstrap percentile interval
           2.5%     97.5% 
       3.947436 11.605769 
      
      		*--------------*

# bootCor() print output is stable

    Code
      print(result)
    Output
      
      	** Bootstrap interval of correlation 
      
       Observed correlation between Icecream$VanillaCalories and Icecream$ChocCalories : 0.98003 
       Mean of bootstrap distribution: 0.97991 
       Standard error of bootstrap distribution: 0.00623 
      
       Bootstrap percentile interval
           2.5%     97.5% 
      0.9661542 0.9902951 
      
      		*--------------*

# bootSlope() print output is stable

    Code
      print(result)
    Output
      
      	** Bootstrap interval of slope 
      
       Observed slope between Icecream$VanillaCalories and Icecream$ChocCalories : 1.0539 
       Mean of bootstrap distribution: 1.05576 
       Standard error of bootstrap distribution: 0.03129 
      
       Bootstrap percentile interval
           2.5%     97.5% 
      0.9959494 1.1166018 
      
      		*--------------*

# permTest() print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: two.sided 
       Observed mean 
        No :  286.6917 	 Yes :  453.7974 
       Observed difference: -167.1057 
      
       Mean of permutation distribution: -1.02484 
       Standard error of permutation distribution: 62.39977 
       P-value:   0.004 
      
      	*-------------*
      

# boot() categorical print output is stable

    Code
      print(result)
    Output
      
      	** Bootstrap interval for difference of proportion 
      
       Observed difference of proportion : A - B = 0.25 
       Mean of bootstrap distribution: 0.2325 
       Standard error of bootstrap distribution: 0.33435 
      
       Bootstrap percentile interval
       2.5% 97.5% 
      -0.50  0.75 
      
      		*--------------*

# permTest() categorical print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: two.sided 
       Observed proportion 
        A :  0.5 	 B :  0.5 
       Observed difference: 0 
      
       Mean of permutation distribution: -0.02705 
       Standard error of permutation distribution: 0.36785 
       P-value:       1 
      
      	*-------------*
      

# boot() penguin proportion print output is stable

    Code
      print(result)
    Output
      
      	** Bootstrap interval for difference of proportion 
      
       Observed difference of proportion : Electronic - Metal = 0.16218 
       Mean of bootstrap distribution: 0.16402 
       Standard error of bootstrap distribution: 0.04676 
      
       Bootstrap percentile interval
            2.5%      97.5% 
      0.08162009 0.25347876 
      
      		*--------------*

# permTest() penguin proportion print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: two.sided 
       Observed proportion 
        Electronic :  0.3597884 	 Metal :  0.1976048 
       Observed difference: 0.16218 
      
       Mean of permutation distribution: -0.00234 
       Standard error of permutation distribution: 0.04836 
       P-value:   0.002 
      
      	*-------------*
      

# permTestPaired() print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: two.sided 
       Observed mean
        Icecream$VanillaCalories :  191.4103 	 Icecream$ChocCalories :  198.7436 
       Observed difference: 7.33333 
      
       Mean of permutation distribution: 0.05513 
       Standard error of permutation distribution: 2.38151 
       P-value:   0.002 
      
      	*-------------*
      

# permTestAnova() print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: greater 
       Observed F statistic: 15.3648 
       Mean of permutation distribution: 1.05676 
       Standard error of permutation distribution: 0.70723 
       P-value:   0.001 
      
      	*-------------*
      

# permTestCor() print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: two.sided 
       Observed correlation between Icecream$VanillaCalories ,  Icecream$ChocCalories : 0.98 
       Mean of permutation distribution: 0.00089 
       Standard error of permutation distribution: 0.16395 
       P-value:   0.001 
      
      	*-------------*
      

# permTestSlope() print output is stable

    Code
      print(result)
    Output
      
      	** Permutation test **
      
       Permutation test with alternative: two.sided 
       Observed slope between Icecream$VanillaCalories ,  Icecream$ChocCalories :  1.0539 
       Mean of permutation distribution: 0.00096 
       Standard error of permutation distribution: 0.17631 
       P-value:   0.001 
      
      	*-------------*
      

# anovaSummarized() output is stable

    Code
      anovaSummarized(N, mn, stdev)
    Output
                                          
      Treatment SS            231129.16210
      Residual SS             195556.02100
      numerator DF                 5.00000
      denominator DF              65.00000
      Residual standard error     54.85029
      
           F-stat     P-value 
      1.53648e+01 5.93642e-10 

# summary.carlboot() output is stable

    Code
      summary(result)
    Output
      Replications: 500 
      
      Summary Statistics of Bootstrap Distribution: 
        Observed     Mean        SE
      1 18.81333 18.83433 0.9604015

# summary.carlperm() output is stable

    Code
      summary(result)
    Output
      Replications: 999 
      
      Summary Statistics of Permutation Distribution: 
      
         Observed     Mean Alternative P.value
      1 -167.1057 -1.02484   two.sided   0.004

# summary.carlperm() includes df columns for ANOVA results

    Code
      summary(result)
    Output
      Replications: 999 
      
      Summary Statistics of Permutation Distribution: 
      
        Observed     Mean df1 df2 P.value
      1  15.3648 1.056757   5  65   0.001

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootPaired.R, R/bootPaired.default.R,
%   R/bootPaired.formula.R
\name{bootPaired}
\alias{bootPaired}
\alias{bootPaired.default}
\alias{bootPaired.formula}
\title{Bootstrap paired data}
\usage{
bootPaired(x, ...)

\method{bootPaired}{default}(
  x,
  y,
  conf.level = 0.95,
  B = 10000,
  plot.hist = TRUE,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  plot.qq = FALSE,
  x.name = deparse(substitute(x)),
  y.name = deparse(substitute(y)),
  seed = NULL,
  ...
)

\method{bootPaired}{formula}(formula, data, subset, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{y}{a numeric vector.}

\item{conf.level}{confidence level for the bootstrap percentile interval.}

\item{B}{number of resamples (positive integer greater than 2).}

\item{plot.hist}{logical. If \code{TRUE}, plot the histogram of the
bootstrap distribution.}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{plot.qq}{logical. If \code{TRUE}, a normal quantile-quantile plot of
the replicates will be created.}

\item{x.name}{Label for variable x}

\item{y.name}{Label for variable y}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula \code{y ~ x} where \code{x, y} are both numeric vectors}

\item{data}{a data frame that contains the variables given in the formula.}

\item{subset}{an optional expression indicating what observations to use.}
}
\value{
The command returns a vector with the replicates of the
statistic being bootstrapped.
}
\description{
Perform a bootstrap of two paired variables.
}
\details{
The command will compute the difference of \code{x} and \code{y} and
bootstrap the difference. The mean and standard error of the bootstrap
distribution will be printed as well as a bootstrap percentile interval.

Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{bootPaired(default)}: Perform a bootstrap of two paired variables.

\item \code{bootPaired(formula)}: Perform a bootstrap of two paired variables.

}}
\examples{

#Bootstrap the mean difference of fat content in vanilla and chocolate ice
#cream. Data are paired becaues ice cream from the same manufacturer will
#have similar content.
Icecream
bootPaired(ChocFat ~ VanillaFat, data = Icecream)
bootPaired(Icecream$VanillaFat, Icecream$ChocFat)

}
\references{
Tim Hesterberg's website
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Laura Chihara
}
\keyword{bootstrap}
\keyword{randomization}
\keyword{resampling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confint.R
\name{confint.carlboot}
\alias{confint.carlboot}
\title{Calculate a CI from a \code{carlboot} object}
\usage{
\method{confint}{carlboot}(object, parm = NULL, level = 0.95, ...)
}
\arguments{
\item{object}{The carlboot object to print.}

\item{parm}{not used in CarletonStats, just for generic consistency}

\item{level}{the confidence level}

\item{...}{not used}
}
\description{
Calculate percentile confidence intervals for a \code{carlboot} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Data.R
\docType{data}
\name{Icecream}
\alias{Icecream}
\title{Ice cream data}
\format{
A data frame with 39 observations on the following 7 variables.
\describe{
\item{Brand}{Brand name}
\item{VanillaCalories}{Calories per serving in vanilla}
\item{VanillaFat}{Fat per serving (g) in vanilla}
\item{VanillaSugar}{Sugar per serving (g) in vanilla}
\item{ChocCalories}{Calories per serving in chocolate}
\item{ChocFat}{Fat per serving (g) in chocolate}
\item{ChocSugar}{Sugar per serving (g) in chocolate}
}
}
\source{
Data collected by Carleton student Ann Butkowski (2008).
}
\description{
Nutritional information on vanilla and chocolate ice cream from a sample of
companies.
}
\examples{

head(Icecream)
t.test(Icecream$VanillaCalories, Icecream$ChocCalories, paired = TRUE)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permTestCor.R, R/permTestCor.default.R,
%   R/permTestCor.formula.R
\name{permTestCor}
\alias{permTestCor}
\alias{permTestCor.default}
\alias{permTestCor.formula}
\title{Permutation test for the correlation of two variables.}
\usage{
permTestCor(x, ...)

\method{permTestCor}{default}(
  x,
  y,
  B = 999,
  alternative = "two.sided",
  plot.hist = TRUE,
  plot.qq = FALSE,
  x.name = deparse(substitute(x)),
  y.name = deparse(substitute(y)),
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
)

\method{permTestCor}{formula}(formula, data, subset, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{y}{a numeric vector.}

\item{B}{the number of resamples to draw (positive integer greater than 2).}

\item{alternative}{alternative hypothesis. Options are \code{"two.sided"},
\code{"less"} or \code{"greater"}.}

\item{plot.hist}{a logical value. If \code{TRUE}, plot the distribution of
the correlations obtained from each resample.}

\item{plot.qq}{a logical value. If \code{TRUE}, plot the normal
quantile-quantile plot of the correlations obtained from each resample.}

\item{x.name}{Label for variable x}

\item{y.name}{Label for variable y}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula \code{y ~ x} where \code{x, y} are numeric vectors.}

\item{data}{a data frame that contains the variables given in the formula.}

\item{subset}{an optional expression indicating what observations to use.}
}
\value{
Returns invisibly a vector of the correlations obtained by the
randomization.
}
\description{
Hypothesis test for a correlation of two variables. The null hypothesis is
that the population correlation is 0.
}
\details{
Perform a permutation test to test \eqn{latex}{H_0: \rho = 0}, where
\eqn{latex}{\rho}is the population correlation. The rows of the second
variable are permuted and the correlation is re-computed.

The mean and standard error of the permutation distribution is printed as
well as a P-value.

Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{permTestCor(default)}: Permutation test for the correlation of two variables.

\item \code{permTestCor(formula)}: Permutation test for the correlation of two variables.

}}
\examples{

plot(states03$HSGrad, states03$TeenBirths)
cor(states03$HSGrad, states03$TeenBirths)

permTestCor(states03$HSGrad, states03$TeenBirths)
permTestCor(TeenBirths ~ HSGrad, data = states03)

}
\references{
Tim Hesterberg's website:
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Laura Chihara
}
\keyword{correlation}
\keyword{permutation}
\keyword{randomization}
\keyword{resampling}
\keyword{test}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot.R, R/boot.default.R, R/boot.formula.R
\name{boot}
\alias{boot}
\alias{boot.default}
\alias{boot.formula}
\title{Bootstrap}
\usage{
boot(x, ...)

\method{boot}{default}(
  x,
  group = NULL,
  statistic = mean,
  success = NULL,
  conf.level = 0.95,
  B = 10000,
  plot.hist = TRUE,
  plot.qq = FALSE,
  x.name = deparse(substitute(x)),
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
)

\method{boot}{formula}(formula, data, subset, ...)
}
\arguments{
\item{x}{a numeric, logical, factor, or character vector. Logical, factor,
and character vectors with exactly two unique values are converted to 0/1,
and \code{mean} is used to compute the proportion.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{group}{an optional grouping variable (vector), usually a factor
variable. If it is a binary numeric variable, it will be coerced to a
factor.}

\item{statistic}{function that computes the statistic of interest. Default
is the \code{mean}.}

\item{success}{a character string naming the level of \code{x} to code as
1 when \code{x} is a logical, factor, or character variable. Defaults to
\code{NULL}, which uses the second factor level (alphabetically) or
\code{TRUE} for logical vectors.}

\item{conf.level}{confidence level for the bootstrap percentile interval.
Default is 95\%.}

\item{B}{number of times to resample (positive integer greater than 2).}

\item{plot.hist}{logical value. If \code{TRUE}, plot the histogram of the
bootstrap distribution.}

\item{plot.qq}{Logical value. If \code{TRUE}, create a normal
quantile-quantile plot of the bootstrap distribution.}

\item{x.name}{Label for variable name}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula \code{y ~ g} where \code{y} is a numeric vector and
\code{g} a factor variable with two levels. If \code{g} is a binary numeric
vector, it will be coerced to a factor variable. For a single numeric variable,
formula may also be \code{~ y}.}

\item{data}{a data frame that contains the variables given in the formula.}

\item{subset}{an optional expression indicating what observations to use.}
}
\value{
A vector with the resampled statistics is returned invisibly.
}
\description{
Bootstrap a single variable or a grouped variable
}
\details{
Perform a bootstrap of a statistic applied to a single variable, or to the
difference of the statistic computed on two samples (using the grouping
variable). If \code{x} is a binary vector of 0's and 1's and the function is
the mean, then the statistic of interest is the proportion.

Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{boot(default)}: Bootstrap a single variable or a grouped variable

\item \code{boot(formula)}: Bootstrap a single variable or a grouped variable

}}
\examples{

#ToothGrowth data (supplied by R)
#bootstrap mean of a single numeric variable
boot(ToothGrowth$len)

#bootstrap difference in mean of tooth length for two groups.
boot(ToothGrowth$len, ToothGrowth$supp, B = 1000)

#same as above using formula syntax
boot(len ~ supp, data = ToothGrowth, B = 1000)

# Penguin Survival proportion
boot(penguin_survival$Status, data = penguin_survival, B = 1000)

# same as above, but with the formula syntax
boot(~Status, data = penguin_survival, B = 1000)

# Penguin Survival if tagged vs. untagged
# bootstrap difference in proportions of survival
boot(penguin_survival$Status, penguin_survival$TagType, B = 1000)

# now using the formula syntax
boot(Status ~ TagType, data = penguin_survival, B = 1000)

}
\references{
Tim Hesterberg's website
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Laura Chihara
}
\keyword{bootstrap}
\keyword{randomization}
\keyword{resampling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.carlboot}
\alias{plot.carlboot}
\alias{plot.carlperm}
\title{Plot the bootstrap distribution in \code{carlboot} object}
\usage{
\method{plot}{carlboot}(x, bins = 15, size = 5, xlab = NULL, ylab = NULL, title = NULL, ...)

\method{plot}{carlperm}(x, bins = 15, size = 5, xlab = NULL, ylab = NULL, title = NULL, ...)
}
\arguments{
\item{x}{The carlboot object to print.}

\item{bins}{number of bins in histogram.}

\item{size}{size of points.}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{...}{not used}
}
\description{
Plot the bootstrap distribution returned as a \code{carlboot} object.
}
\examples{

boot_dist <- boot(ToothGrowth$len, ToothGrowth$supp, B = 1000)
plot(boot_dist)

perm_dist <- permTest(states03$ViolentCrime, states03$DeathPenalty, B = 999)
plot(perm_dist)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stemPlot.R, R/stemPlot.default.R,
%   R/stemPlot.formula.R
\name{stemPlot}
\alias{stemPlot}
\alias{stemPlot.default}
\alias{stemPlot.formula}
\title{Stem and leaf plot}
\usage{
stemPlot(x, ...)

\method{stemPlot}{default}(x, grpvar = NULL, varname = NULL, grpvarname = NULL, ...)

\method{stemPlot}{formula}(formula, data = parent.frame(), subset, ...)
}
\arguments{
\item{x}{a numeric variable.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{grpvar}{a factor variable. A stem plot of \code{x} will be created for
each level of the factor variable.}

\item{varname}{name of the numeric variable. This is for printing the output
only. Change if you want to print out a name different from the actual
variable name.}

\item{grpvarname}{name of the factor variable.  This is for printing the
output only. Change if you want to print out a name different from the
actual variable name.}

\item{formula}{a formula of the form \code{x ~ grpvar} where \code{x} is
numeric and \code{grpvar} is a factor variable.}

\item{data}{a data frame with the variables in the formula.}

\item{subset}{an optional expression specifying which observations to keep.}
}
\description{
Stem and leaf plot. Will accept a factor variable as a second argument to
create stem plots for each of the levels.
}
\details{
This command is just an enhanced version of R's \code{stem} command. It
allows the user to create the stem plot for a numeric variable grouped by
the levels of a factor variable.
}
\section{Methods (by class)}{
\itemize{
\item \code{stemPlot(default)}: Stem and leaf plot

\item \code{stemPlot(formula)}: Stem and leaf plot

}}
\examples{


stemPlot(states03$Births, states03$Region)

stemPlot(Births ~ Region, data = states03)

}
\author{
Laura Chihara
}
\keyword{plot}
\keyword{stem}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.carlboot}
\alias{summary.carlboot}
\alias{summary.carlperm}
\title{Print a summary of an \code{carlboot} object}
\usage{
\method{summary}{carlboot}(object, ...)

\method{summary}{carlperm}(object, ...)
}
\arguments{
\item{object}{The carlboot object to print.}

\item{...}{not used}
}
\description{
Print summary statistics and confidence intervals, if desired, for an \code{lmeresamp} object.
}
\examples{
boot_dist <- boot(ToothGrowth$len, ToothGrowth$supp, B = 1000)
summary(boot_dist)
perm_dist <- permTest(states03$ViolentCrime, states03$DeathPenalty, B = 999)
summary(perm_dist)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootSlope.R, R/bootSlope.default.R,
%   R/bootSlope.formula.R
\name{bootSlope}
\alias{bootSlope}
\alias{bootSlope.default}
\alias{bootSlope.formula}
\title{Bootstrap the slope of a simple linear regression line}
\usage{
bootSlope(x, ...)

\method{bootSlope}{default}(
  x,
  y,
  conf.level = 0.95,
  B = 10000,
  plot.hist = TRUE,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  plot.qq = FALSE,
  x.name = deparse(substitute(x)),
  y.name = deparse(substitute(y)),
  seed = NULL,
  ...
)

\method{bootSlope}{formula}(formula, data, subset, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{y}{a numeric vector.}

\item{conf.level}{confidence level for the bootstrap percentile interval.}

\item{B}{number of times to resample (positive integer greater than 2).}

\item{plot.hist}{a logical value. If \code{TRUE}, plot the bootstrap
distribution of the resampled slope.}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{plot.qq}{a logical value. If \code{TRUE} a normal quantile-quantile
plot of the bootstraped values is created.}

\item{x.name}{Label for variable x}

\item{y.name}{Label for variable y}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula of the form lhs ~ rhs where lhs is a numeric variable
giving the data values and rhs a factor with two levels giving the corresponding groups.}

\item{data}{an optional data frame containing the variables in the formula formula.
By default the variables are taken from environment(formula).}

\item{subset}{an optional vector specifying a subset of observations to be used.}
}
\value{
The command returns the slopes of the resampled
observations.
}
\description{
Bootstrap theslope of a simple linear regression line. The bootstrap mean and
standard error are printed as well as a bootstrap percentile confidence
interval.
}
\details{
Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{bootSlope(default)}: Bootstrap the slope of a simple linear regression line

\item \code{bootSlope(formula)}: Bootstrap the slope of a simple linear regression line

}}
\examples{


plot(states03$ColGrad, states03$InfMortality)
bootSlope(InfMortality ~ ColGrad, data = states03, B = 1000)
bootSlope(states03$ColGrad, states03$InfMortality, B = 1000)

}
\references{
Tim Hesterberg's website
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Adam Loy, Laura Chihara
}
\keyword{randomiziation}
\keyword{resampling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupedBar.R, R/groupedBar.default.R,
%   R/groupedBar.formula.R
\name{groupedBar}
\alias{groupedBar}
\alias{groupedBar.default}
\alias{groupedBar.formula}
\title{Grouped bar chart}
\usage{
groupedBar(resp, ...)

\method{groupedBar}{default}(
  resp,
  condvar = NULL,
  percent = TRUE,
  print = TRUE,
  cond.name = deparse(substitute(condvar)),
  resp.name = deparse(substitute(resp)),
  ...
)

\method{groupedBar}{formula}(formula, data = parent.frame(), subset, ...)
}
\arguments{
\item{resp}{a factor variable. If \code{resp} is numeric, it will be coerced to a
factor variable.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{condvar}{a factor variable to condition on. If \code{NULL}, then a bar
plot of just the \code{resp} variable will be created. If \code{condvar} is
numeric, it will be coerced to a factor variable.}

\item{percent}{a logical value. Should the y-axis give percent or counts?}

\item{print}{a logical value. If \code{TRUE}, print out the table.}

\item{cond.name}{Label for variable \code{condvar}.}

\item{resp.name}{Label for variable \code{resp}.}

\item{formula}{a formula of the form \code{x ~ condvar}. If \code{x} or
\code{condvar} is (are) not a factor variable, then it (they) will be
coerced into one. Formula can also be \code{ ~ x} for a single factor variable.}

\item{data}{a data frame that contains the variables in the formula.}

\item{subset}{an optional vector specifying a subset of observations to be
used.}
}
\value{
Returns invisibly a table of the variable(s).
}
\description{
Create a bar chart of a single categorical variable or a grouped bar chart
of two categorical variables.
}
\details{
For a single factor variable, a bar plot. If two factor variables are given,
then a bar plot of \code{x} conditioned by \code{condvar}.  This command
uses R's \code{table} command so missing values are automatically removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{groupedBar(default)}: Grouped bar chart

\item \code{groupedBar(formula)}: Grouped bar chart

}}
\examples{


groupedBar(states03$Region)

\dontrun{
groupedBar(states03$DeathPenalty, states03$Region, legend.loc = "topleft")

#Using a formula syntax:

groupedBar(~Region, data = states03)
groupedBar(DeathPenalty ~ Region, data = states03, legend.loc = "topleft")
}

}
\author{
Laura Chihara
}
\keyword{bar}
\keyword{grouped}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/missingLevel.R
\name{missingLevel}
\alias{missingLevel}
\title{Missing observations in factors}
\usage{
missingLevel(data)
}
\arguments{
\item{data}{a data frame with factor variables.}
}
\value{
Returns the same data frame with \code{""""} replaced by \code{<NA>}
in factor variables.
}
\description{
In data frames with factor variables, convert any observation with "" into
<NA>.
}
\details{
In a factor variable with the level \code{""""}, this command will convert
this to an \code{<NA>}.
}
\note{
When importing data from comma separated files (for example), missing
values in a categorical variable are often denoted by """. We often do not
want to treat this as a level of a factor variable in R.
}
\author{
Laura Chihara
}
\keyword{missing}
\keyword{values}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permTestPaired.R, R/permTestPaired.default.R,
%   R/permTestPaired.formula.R
\name{permTestPaired}
\alias{permTestPaired}
\alias{permTestPaired.default}
\alias{permTestPaired.formula}
\title{Permutation test for paired data.}
\usage{
permTestPaired(x, ...)

\method{permTestPaired}{default}(
  x,
  y,
  B = 9999,
  alternative = "two.sided",
  plot.hist = TRUE,
  plot.qq = FALSE,
  x.name = deparse(substitute(x)),
  y.name = deparse(substitute(y)),
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
)

\method{permTestPaired}{formula}(formula, data, subset, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{y}{a numeric vector.}

\item{B}{the number of resamples.}

\item{alternative}{the alternative hypothesis. Options are
\code{"two.sided"}, \code{"less"} and \code{"greater"}.}

\item{plot.hist}{a logical value. If \code{TRUE}, create a histogram
displaying the permutation distribution of the statistic.}

\item{plot.qq}{a logical value. If \code{TRUE}, include a quantile-normal
plot of the permuation distribution.}

\item{x.name}{Label for x variable}

\item{y.name}{Label for y variable}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula of the form \code{y ~ x}, where \code{x, y} are both numeric variables.}

\item{data}{an optional data frame containing the variables in the formula. By default the
variables are taken from environment(formula).}

\item{subset}{an optional vector specifying a subset of observations to be used.}
}
\value{
Returns invisibly a vector of the replicates of the test statistic
(ex. mean of the difference of the resampled variables).
}
\description{
Permutation test for paired data.
}
\details{
For two paired numeric variables with n rows, randomly select k of the n
rows (k also is randm) and switch the entries \eqn{latex}{(a, b) \to (b, a)}
and then compute the mean of the difference of the two variables (\code{y-x}).

Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{permTestPaired(default)}: Permutation test for paired data.

\item \code{permTestPaired(formula)}: Permutation test for paired data.

}}
\examples{

#Does chocolate ice cream have more calories than vanilla ice cream, on average?
#H0: mean number of calories is the same
#HA: mean number of calories is greater in chocolate ice cream

permTestPaired(Icecream$VanillaCalories, Icecream$ChocCalories, alternative = "less")
permTestPaired(ChocCalories ~ VanillaCalories, data = Icecream, alternative = "greater")

}
\references{
Tim Hesterberg's website:
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Laura Chihara
}
\keyword{data}
\keyword{paired}
\keyword{permutation}
\keyword{randomization}
\keyword{resampling}
\keyword{test}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corDemo.R
\name{corDemo}
\alias{corDemo}
\title{Correlation demonstration}
\usage{
corDemo(r = 0)
}
\arguments{
\item{r}{a number between -1 and 1. Enter any number r, \eqn{latex}{abs(r)>
1 }, to exit the interactive session[}
}
\description{
For a given \code{r}, create a scatterplot of two variables with that
correlation.
}
\details{
Demonstrate the concept of correlation by inputting a number between -1 and
1 and seeing a scatter plot of two variables with that correlation. Once you
invoke this command, you can continue to enter values for r. Type any number
\eqn{latex}{abs(r)> 1 }) to exit.
}
\examples{

\dontrun{
corDemo()
}

}
\author{
Laura Chihara
}
\keyword{Correlation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confIntDemo.R
\name{confIntDemo}
\alias{confIntDemo}
\title{Confidence Interval Demonstration}
\usage{
confIntDemo(distr = "normal", size = 20, conf.level = 0.95)
}
\arguments{
\item{distr}{distribution of the population to be sampled. Options include
\code{"normal"}, \code{"exponential"}, \code{"uniform"} and \code{"binary"}
(partial match allowed).}

\item{size}{sample size}

\item{conf.level}{confidence level.}
}
\value{
The command invisibly returns the fraction of intervals that capture
the true mean.
}
\description{
Draw many random samples and compute confidence interval.  How many
intervals capture the true mean?
}
\details{
This simulation will draw 100 random samples from a given population
distribution and compute the correpsonding confidence intervals. The 100
intervals will be drawn with an indication of the ones that missed the true
mean. A histogram of the population will also be created.
}
\examples{

confIntDemo()

confIntDemo(distr = "exponential", size = 40)


}
\author{
Laura Chihara
}
\keyword{confidence}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CarletonStats-package.R
\docType{package}
\name{CarletonStats-package}
\alias{CarletonStats}
\alias{CarletonStats-package}
\title{CarletonStats: Functions for Statistics Classes at Carleton College}
\description{
Includes commands for bootstrapping and permutation tests, a command for created grouped bar plots, and a demo of the quantile-normal plot for data drawn from different distributions.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/aloy/CarletonStats}
  \item Report bugs at \url{https://github.com/aloy/CarletonStats/issues}
}

}
\author{
\strong{Maintainer}: Adam Loy \email{aloy@carleton.edu} (\href{https://orcid.org/0000-0002-5780-4611}{ORCID})

Authors:
\itemize{
  \item Adam Loy \email{aloy@carleton.edu} (\href{https://orcid.org/0000-0002-5780-4611}{ORCID})
  \item Laura Chihara \email{flchihara@carleton.edu}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qqPlotDemo.R
\name{qqPlotDemo}
\alias{qqPlotDemo}
\title{Demonstration of the normal qq-plot.}
\usage{
qqPlotDemo(
  n = 25,
  distribution = "normal",
  mu = 0,
  sigma = 1,
  df = 10,
  lambda = 10,
  numdf = 10,
  dendf = 16,
  shape1 = 40,
  shape2 = 5
)
}
\arguments{
\item{n}{sample size}

\item{distribution}{population distribution. Options are \code{"normal"},
\code{"t"},\code{"exponential"}, \code{"chi.square"}, \code{"F"} or
\code{"beta"} (partial matches are accepted).}

\item{mu}{mean for the normal distribution.}

\item{sigma}{(positive) standard deviation for the normal distribution.}

\item{df}{(positive) degrees of freedom for the t-distribution.}

\item{lambda}{positive rate for the exponential distribution.}

\item{numdf}{(positive) numerator degrees of freedom for the chi-square
distribution.}

\item{dendf}{(positive) denominator degrees of freedom for the chi-square
distribution.}

\item{shape1}{positive parameter for the beta distribution (shape1 = a).}

\item{shape2}{positive parameter for the beta distribution (shape2 = b).}
}
\value{
Returns invisibly the random sample.
}
\description{
Demonstrate the normal quantile-quantile plot for samples drawn from
different populations.
}
\details{
Draw a random sample from the chosen sample and display the normal qq-plot
as well as the histogram of its distribution.
}
\examples{

qqPlotDemo(n = 30, distr = "exponential", lambda = 1/3)


}
\author{
Laura Chihara
}
\keyword{normal}
\keyword{plot}
\keyword{quantile-quantile}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.carlboot}
\alias{print.carlboot}
\alias{print.carlperm}
\title{Print a summary of an \code{carlboot} object}
\usage{
\method{print}{carlboot}(x, ...)

\method{print}{carlperm}(x, ...)
}
\arguments{
\item{x}{The carlboot object to print.}

\item{...}{not used}
}
\description{
Print summary statistics and confidence intervals for an \code{carlboot} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Data.R
\docType{data}
\name{states03}
\alias{states03}
\title{US government data, 2003}
\format{
A data frame with 50 observations on the following 24 variables.
\describe{
\item{State}{the 50 states}
\item{Region}{a
factor with levels \code{Midwest}, \code{Northeast}, \code{South},
\code{West}}
\item{Pop}{Population in 1000}
\item{Births}{Number of births}
\item{Deaths}{Number of
deaths}
\item{Pop18}{Percent of population 18 years of age or
younger}
\item{Pop65}{Percent of population 65 years of age or
older}
\item{HSGrad}{Percent of population 25 years of age or older
with a high school degree}
\item{ColGrad}{Percent of population 25
years of age or older with a college degree}
\item{TeacherPay}{Average teachers salary in dollars}
\item{InfMortality}{Infant mortality per 1000 live births}
\item{TeenBirths}{Live births per 1000 15-19 year old females}
\item{ViolentCrime}{Violent crime per 100000 population}
\item{PropertyCrime}{Property crime per 100000 population}
\item{DeathPenalty}{State has death penalty?}
\item{Executions}{Number of executions 1977-2003}
\item{Poverty}{Percent of populaton below the poverty level}
\item{Unemp}{Percent unemployed (of population 16 years or older)}
\item{Uninsured}{Percent uninsured (3 year aveage)}
\item{Income}{Median household income in 1998 dollars}
\item{Earnings}{Average hourly earnings of production workers in
manufacturing}
\item{Heart}{Deaths by heart disease per 100000
population}
\item{Vehicles}{Deaths by motor vehicle accidents per
100000 population}
 \item{Homeowners}{Home ownership rate}
}
}
\source{
United States Census Bureau \url{https://www.census.gov/}
}
\description{
Census data on the 50 states from 2003.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Data.R
\docType{data}
\name{penguin_survival}
\alias{penguin_survival}
\title{Penguin survival by tag type}
\format{
A data frame with 2 observations on the following 3 variables.
\describe{
  \item{TagType}{Tag type: \code{Metal} or \code{Electronic}}
  \item{Status}{Survival status: \code{Survived} or \code{Electronic}}
}
}
\source{
Saraux, C., et al., “Reliability of flipper-banded penguins as
indicators of climate change,” Nature, January 2011; 469: 203–206.

Lock, Robin H., et al. Statistics: Unlocking the power of data.
John Wiley & Sons, 2025.
}
\description{
Survival outcomes for penguins fitted with metal or electronic tags.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permTestAnova.R, R/permTestAnova.default.R,
%   R/permTestAnova.formula.R
\name{permTestAnova}
\alias{permTestAnova}
\alias{permTestAnova.default}
\alias{permTestAnova.formula}
\title{Permutation test for ANOVA F-test}
\usage{
permTestAnova(x, ...)

\method{permTestAnova}{default}(
  x,
  group,
  B = 9999,
  plot.hist = TRUE,
  plot.qq = FALSE,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
)

\method{permTestAnova}{formula}(formula, data = parent.frame(), subset, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{group}{a factor variable with two or more levels. If \code{group} is a
numeric vector, it will be coerced into a factor variable.}

\item{B}{the number of resamples (positive integer greater than 2).}

\item{plot.hist}{a logical value. If \code{TRUE}, the permutation
distribution of the statistic is plotted.}

\item{plot.qq}{a logical value. If \code{TRUE}, then a normal
quantile-quantile plot of the resampled test statistic is created.}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula of the form \code{y ~ group} where \code{y} is
numeric and \code{group} is a factor variable.}

\item{data}{a data frame with the variables in the formula.}

\item{subset}{an optional expression specifying which observations to keep.}
}
\value{
Returns invisibly a vector of the replicates of the test statistic.
}
\description{
Permutation test to see if the population mean is the same for two or more
populations. For instance, test \eqn{latex}{H_0: \mu_1 = \mu_2 = \mu_3} where
\eqn{latex}{\mu} denotes the population mean. The values of the numeric
variable are randomly assigned to the groups and the ANOVA F statistic is
calculated. The command will print the mean and
standard error of the distribution of the test statistic as well as a
P-value.
}
\details{
Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{permTestAnova(default)}: Permutation test for ANOVA F-test

\item \code{permTestAnova(formula)}: Permutation test for ANOVA F-test

}}
\examples{

permTestAnova(states03$ViolentCrime, states03$Region, B = 499)

#using formula syntax
\dontrun{
permTestAnova(ViolentCrime ~ Region, data = states03, B = 9999)
}

}
\references{
Tim Hesteberg's website:
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Adam Loy, Laura Chihara
}
\keyword{permutation}
\keyword{randomization}
\keyword{resampling}
\keyword{test}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permTestSlope.R, R/permTestSlope.default.R,
%   R/permTestSlope.formula.R
\name{permTestSlope}
\alias{permTestSlope}
\alias{permTestSlope.default}
\alias{permTestSlope.formula}
\title{Permutation test for the Slope}
\usage{
permTestSlope(x, ...)

\method{permTestSlope}{default}(
  x,
  y,
  B = 999,
  alternative = "two.sided",
  plot.hist = TRUE,
  plot.qq = FALSE,
  x.name = deparse(substitute(x)),
  y.name = deparse(substitute(y)),
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
)

\method{permTestSlope}{formula}(formula, data, subset, ...)
}
\arguments{
\item{x}{a numeric vector.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{y}{a numeric vector.}

\item{B}{the number of resamples to draw (positive integer greater than 2).}

\item{alternative}{alternative hypothesis. Options are \code{"two.sided"},
\code{"less"} or \code{"greater"}.}

\item{plot.hist}{a logical value. If \code{TRUE}, plot the distribution of
the slopes obtained from each resample.}

\item{plot.qq}{a logical value. If \code{TRUE}, plot the normal
quantile-quantile plot of the slopes obtained from each resample.}

\item{x.name}{Label for variable x}

\item{y.name}{Label for variable y}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula \code{y ~ x} where \code{x, y} are numeric vectors.}

\item{data}{a data frame that contains the variables given in the formula.}

\item{subset}{an optional expression indicating what observations to use.}
}
\value{
Returns invisibly a vector of the slopes obtained by the
randomization.
}
\description{
Hypothesis test for a slope of a simple linear regression model.
The null hypothesis is that the population slope is 0.
}
\details{
Perform a permutation test to test \eqn{latex}{H_0: \beta = 0}, where
\eqn{latex}{\beta}is the population slope. The rows of the second
variable are permuted and the slope is re-computed.

The mean and standard error of the permutation distribution is printed as
well as a P-value.

Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{permTestSlope(default)}: Permutation test for the slope

\item \code{permTestSlope(formula)}: Permutation test for the slope

}}
\examples{

plot(states03$HSGrad, states03$TeenBirths)
lm(HSGrad ~ TeenBirths, data = states03)

permTestSlope(states03$HSGrad, states03$TeenBirths)
permTestSlope(TeenBirths ~ HSGrad, data = states03)

}
\references{
Tim Hesterberg's website:
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Adam Loy, Laura Chihara
}
\keyword{correlation}
\keyword{permutation}
\keyword{randomization}
\keyword{resampling}
\keyword{test}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anovaSummarized.R
\name{anovaSummarized}
\alias{anovaSummarized}
\title{Anova F test}
\usage{
anovaSummarized(N, mn, stdev)
}
\arguments{
\item{N}{a vector with the sample sizes}

\item{mn}{a vector of means, one for each group in the sample}

\item{stdev}{a vector of standard deviations, one for each group in the
sample}
}
\value{
Returns invisibly a list

\item{Treatment SS}{The treatment sum of squares (also called the "between
sum of squares").} \item{Residual SS}{Residual sum of squares (also called
the "within sum of squares").} \item{Degrees of Freedom}{a vector with the
numerator and denominator degrees of freedom.} \item{Treatment Mean
Square}{Treatment SS/numerator DF} \item{Residual Mean Square}{Residual
SS/denominator DF} \item{Residual Standard Error}{Square root of Residual
Mean Square} \item{F}{the F statistic} \item{P-value}{p-value} ...
}
\description{
ANOVA F test when given summarized data (sample sizes, means and standard
deviations).
}
\details{
Perform an ANOVA F test when presented with summarized data: sample sizes,
sample means and sample standard devations.
}
\examples{

#use the data set chickwts from base R
head(chickwts)

N <- table(chickwts$feed)
stdev <- tapply(chickwts$weight, chickwts$feed, sd)
mn <- tapply(chickwts$weight, chickwts$feed, mean)

anovaSummarized(N, mn, stdev)

}
\author{
Laura Chihara
}
\keyword{ANOVA}
\keyword{Summarized}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permTest.R, R/permTest.default.R,
%   R/permTest.formula.R
\name{permTest}
\alias{permTest}
\alias{permTest.default}
\alias{permTest.formula}
\title{Permutation test}
\usage{
permTest(x, ...)

\method{permTest}{default}(
  x,
  group,
  statistic = mean,
  success = NULL,
  B = 9999,
  alternative = "two.sided",
  plot.hist = TRUE,
  plot.qq = FALSE,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  seed = NULL,
  ...
)

\method{permTest}{formula}(formula, data = parent.frame(), subset, ...)
}
\arguments{
\item{x}{a numeric, logical, factor, or character vector. Logical, factor,
and character vectors with exactly two unique values are converted to 0/1,
and \code{mean} is used to compute the proportion.}

\item{\dots}{further arguments to be passed to or from methods.}

\item{group}{a factor variable with two levels. If \code{group} is a binary
numeric vector, it will be coerced into a factor variable.}

\item{statistic}{the statistic of interest.}

\item{success}{a character string naming the level of \code{x} to code as
1 when \code{x} is a logical, factor, or character variable. Defaults to
\code{NULL}, which uses the second factor level (alphabetically) or
\code{TRUE} for logical vectors.}

\item{B}{the number of resamples (positive integer greater than 2).}

\item{alternative}{the alternative hypothesis. Options are
\code{"two.sided"}, \code{"less"} or \code{"greater"}.}

\item{plot.hist}{a logical value. If \code{TRUE}, the permutation
distribution of the statistic is plotted.}

\item{plot.qq}{a logical value. If \code{TRUE}, then a normal
quantile-quantile plot of the resampled test statistic is created.}

\item{xlab}{an optional character string for the x-axis label}

\item{ylab}{an optional character string for the y-axis label}

\item{title}{an optional character string giving the plot title}

\item{seed}{optional argument to \code{\link{set.seed}}}

\item{formula}{a formula of the form \code{y ~ group} where \code{y} is
numeric and \code{group} is a factor variable.}

\item{data}{a data frame with the variables in the formula.}

\item{subset}{an optional expression specifying which observations to keep.}
}
\value{
Returns invisibly a vector of the replicates of the test statistic.
}
\description{
Permutation test to test a hypothesis involving two samples.
}
\details{
Permutation test to see if a population parameter is the same for two
populations. For instance, test \eqn{latex}{H_0: \mu_M = \mu_F} where
\eqn{latex}{\mu} denotes the population mean. The values of the numeric
variable are randomly assigned to the two groups and the difference of the
statistic for each group is calculated. The command will print the mean and
standard error of the distribution of the test statistic as well as a
P-value.

Observations with missing values are removed.
}
\section{Methods (by class)}{
\itemize{
\item \code{permTest(default)}: Permutation test

\item \code{permTest(formula)}: Permutation test

}}
\examples{

# Testing the difference in means
permTest(states03$ViolentCrime, states03$DeathPenalty)

#using formula syntax
permTest(ViolentCrime ~ DeathPenalty, data = states03, alt = "less")

# Testing the difference in proportions
permTest(penguin_survival$Status, penguin_survival$TagType, B = 999)

#using formula syntax
permTest(Status ~ TagType, data = penguin_survival, B = 999)

}
\references{
Tim Hesteberg's website:
\url{https://www.timhesterberg.net/bootstrap-and-resampling}
}
\author{
Laura Chihara
}
\keyword{permutation}
\keyword{randomization}
\keyword{resampling}
\keyword{test}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Data.R
\docType{data}
\name{Milkshakes}
\alias{Milkshakes}
\title{Milkshakes (chocolate)
Nutrional information on chocolate milkshakes from a sample of restaurants.}
\format{
A data frame with 29 observations on the following 11 variables.
\describe{
\item{Restaurant}{Names of restaurants}
\item{Type}{Type of restaurant, \code{Dine In} \code{Fast Food}}
\item{Calories}{Calories per serving}
\item{Fat}{Fat per
serving (g)}
\item{Sodium}{Sodium per serving (mg)}
\item{Carbs}{Carbohydrates per serving (g)}
\item{SizeOunces}{Size of milkshake (ounces)}
\item{CalPerOunce}{Calories per ounce}
\item{FatPerOunce}{Fat per ounce}
\item{CarbsPerOunce}{Carbohydrates per ounce}
}
}
\source{
Data collected by Carleton students Yoni Blumberg (2013) and Lindsay
Guthrie (2013).
}
\description{
Milkshakes (chocolate)
Nutrional information on chocolate milkshakes from a sample of restaurants.
}
\keyword{datasets}
